! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

! cchidp.f: subroutines for the calculation of chi^(0) at q=0 and q/=0

!     ==================================================================
      subroutine cchi0q0(p,gvec,op,kibz,
     &   nbv,occ,ktab,ktabr,ktabi,ktabo,kbz,
     &   irottb,nfft1,nfft1a,nfft2,nfft3,igfft,nr,
     &   energy,gwenergy,wfg,wfr,chi0,lwing,uwing,
     &   b1,b2,b3,ucvol,wtk,grottb,
     &   inclvkb,kb)
!     ==================================================================
!     calculate chi0 in the limit q->0
!     Note: the terms "head", "wings" and "body" of Chi(G,G') refer to
!           G=G'=0, either G or G' =0, and neither=0 respectively
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      type(dpparam) p
      integer nr, nrb
      integer nfft1, nfft2, nfft3, nfft1a
      complex chi0(p%npwmat,p%npwmat,p%nomega)
      complex lwing(p%npwmat,p%nomega,p%nq)
      complex uwing(p%npwmat,p%nomega,p%nq)
      integer igfft(p%npwvec,5,5,5)
      integer gvec(3,p%npwvec)
      real energy(p%nkibz,p%nbnds)
      real gwenergy(p%nkibz,p%nbnds)
      real occ(p%nkibz,p%nbnds)
      complex wfg(p%npwwfn,p%nbnds,p%nkibz)
      complex wfr(nr,p%nbnds,p%nkibz)
!      complex gradvnl(3,p%npwwfn,p%npwwfn,p%nkibz)
      type(kb_potential) kb
      double precision factocc
      integer nbv
      real op(3,3,p%nop)
      real opinv(3,3)
      integer irottb(nr,p%nop), grottb(p%npwvec,2,p%nop)
      real kibz(3,p%nkibz), kbz(3,p%nkbz)
      integer ktab(p%nkbz), ktabr(nr,p%nkbz), ktabi(p%nkbz)
      integer ktabo(p%nkbz)
      real wtk(p%nkibz)
      real b1(3), b2(3), b3(3)
      real ucvol
      
      logical inclvkb

      complex, external :: pdtqrc

      real qrot(3)
      complex rhotwx(3)
      complex ct
      complex cta1,cta2(3),cta3(3),cta4
      complex (kind(0.0d0)) :: den(p%nomega)
      complex (kind(0.0d0)) :: ediff, egwdiff
      complex :: wfg1(p%npwwfn),wfg2(p%npwwfn)
      complex :: fnlkslm(p%npwwfn),fnldkslm(3,p%npwwfn)
      integer ikbz, ikibz, iop, ik, iinv, ibv, ibc, ig, igp, ir, iq
      integer ilm, is
      integer istat
      real weight

      complex :: rhotwg(p%npwmat)
      complex :: rhotw(nr)

      logical :: NOANTIRESONANT = .false.
      logical, parameter :: ONLYANTIRESONANT = .false.
      logical ltemp

      print *, 'calculating chi0 at q=0'
      inquire(file='chiparam.in',exist=ltemp)
      if(ltemp) then
        open(90,file='chiparam.in')
        read(90,*) NOANTIRESONANT
        close(90)
      endif

      if(NOANTIRESONANT) then
        print *, 'WARNING: antiresonant term excluded'
      endif
      if(ONLYANTIRESONANT) then
        print *, 'WARNING: resonant term excluded'
      endif

      nrb = nfft1 * nfft2 * nfft3
      weight = 2.0 / p%nkbz

!     zero chi0
      chi0(:,:,:) = 0.0
      lwing(:,:,:) = 0.0
      uwing(:,:,:) = 0.0

!     epscp(:,:) = 0.0

!     loop over k-points in BZ
      do ikbz = 1, p%nkbz
        write(unit=6,fmt='(t3,"status:",i4,"/",i5," completed")',
     &        advance='no') ikbz, p%nkbz
        ikibz = ktab(ikbz)
        iop = ktabo(ikbz)
        iinv = (3-ktabi(ikbz))/2

!       the wavefunctions for the k-point in the bz are:
!       wfn(G,b,kbz) = wfn(R^-1G,b,kibz) ( = wfn*(R^-1G,b,kibz) for inversion)
!       wfn(r,b,kbz) = wfn(R^-1r,b,kibz) ( = wfn*(R^-1r,b,kibz) for inversion)
!       the gradient of the Vnl(K,K') for the k-point in the bz should be:
!       gradvnl(G,G',kbz) = R gradvnl(R^-1G,R^-1G',kibz)

!       now loop over bands
        do ibv = p%lomo, p%nbnds
          do ibc = p%lomo, p%nbnds
            factocc = 0.5 * (occ(ikibz,ibc) - occ(ikibz,ibv))
!           we continue the loop only if factocc != 0
            if(abs(factocc) < 0.01) cycle
            if(NOANTIRESONANT .and. factocc < 0.0) cycle
            if(ONLYANTIRESONANT .and. factocc > 0.0) cycle

            ediff = energy(ikibz,ibc) - energy(ikibz,ibv)
            egwdiff = gwenergy(ikibz,ibc) - gwenergy(ikibz,ibv)
            den(:)=factocc/(p%omega(:)+egwdiff)

!           calculate rho-twiddle(G)
            call rho_tw_g(p%npwmat,nr,nrb,nfft1a,nfft1,nfft2,nfft3,
     &        igfft(:,3,3,3),wfr(:,ibc,ikibz),iinv,ktabr(:,ikbz),
     &        wfr(:,ibv,ikibz),iinv,ktabr(:,ikbz),rhotwg)

!           set up rho-twiddle(G=0) using small vector q instead of zero
!           and k.p perturbation theory
            wfg1(:)=wfg(:,ibv,ikibz)
            wfg2(:)=wfg(:,ibc,ikibz)
            rhotwx(:) = 0
!           first term <c,k|-iq*grad|v,k>
            do ig = 1, p%npwwfn
              ct = conjg(wfg2(ig)) * wfg1(ig)
              rhotwx(:) = rhotwx(:) + gvec(:,ig) * ct
            enddo
!           second term <c,k|[Vnl,iqr]|v,k>
            if(inclvkb) then
!              non-separable form
!              do ig = 1, p%npwwfn
!                do igp = 1, p%npwwfn
!                  ct = conjg(wfg(ig,ibc,ikibz)) * wfg(igp,ibv,ikibz)
!                 must multiply by ucvol because yet gradvnl contain 1/ucvol
!                  rhotwx(:) = rhotwx(:) + ct * gradvnl(:,ig,igp,ikibz)
!                 enddo
!              enddo
!             separable form
              do is=1, kb%natom
               do ilm = 1, kb%mpsang*kb%mpsang
                cta1=0
                cta2=0
                cta3=0
                cta4=0
                fnlkslm(:)=kb%fnl(:,ilm,is,ikibz)
                fnldkslm(:,:)=kb%fnld(:,:,ilm,is,ikibz)
                do ig = 1, p%npwwfn
                 cta1 = cta1 + wfg2(ig)*fnlkslm(ig)
                 cta2(:) = cta2(:) + wfg1(ig)*fnldkslm(:,ig)
                 cta3(:) = cta3(:) + wfg2(ig)*fnldkslm(:,ig)
                 cta4 = cta4 + wfg1(ig)*fnlkslm(ig)
                enddo
                rhotwx(:)=rhotwx(:)
     &                 +conjg(cta1)*cta2(:)+conjg(cta3(:))*cta4
               enddo
              enddo
            endif
            rhotwx(:) = - rhotwx(:) / ediff

!           rhotwg(1) = R^-1q * rhotwx_ibz
!           rhotwg(1) = -R^-1q * conjg(rhotwx_ibz) for inversion
            opinv(:,:) = op(:,:,iop)
            call rginv(3,opinv,3)
            call dosym(opinv,iinv,p%q(:,1),qrot)
            rhotwg(1) = pdtqrc(qrot,rhotwx,b1,b2,b3)
            if(iinv == 2) rhotwg(1) = conjg(rhotwg(1))

            call assemblychi0(p%npwmat,p%nomega,chi0,rhotwg,den)

            do iq = 1, p%nq
              call dosym(opinv,iinv,p%q(:,iq),qrot)
              rhotwg(1) = pdtqrc(qrot,rhotwx,b1,b2,b3)
              if(iinv == 2) rhotwg(1) = conjg(rhotwg(1))
                
              call assemblywings(p%npwmat,p%nomega,
     &                     lwing(:,:,iq),uwing(:,:,iq),rhotwg,den)
            enddo
                
!           rhotwxc(:)=b1(:)*rhotwx(1)+b2(:)*rhotwx(2)+b3(:)*rhotwx(3)
!	      do io = 1, nomega
!	        epscp(:,io) = epscp(:,io) + 1.- (4.*PI/ucvol)*
!     &          real(rhotwxc(:)*conjg(rhotwxc(:)))*den(io)*(2./p%nkbz)
!	      enddo

          enddo ! ibc
        enddo ! ibv
      enddo ! ikbz

      weight = 2.0 / p%nkbz
      chi0(:,:,:) = weight * chi0(:,:,:) / ucvol
      lwing(:,:,:) = weight * lwing(:,:,:) / ucvol
      uwing(:,:,:) = weight * uwing(:,:,:) / ucvol

      print *
      print *
!     now we have chi0

      return
      end



!     ==================================================================
      subroutine cchi0(p,q,
     &   nbv,occ,ktab,ktabr,ktabi,kbz,
     &   nfft1,nfft1a,nfft2,nfft3, igfft, nr,
     &   energy,gwenergy,wfr,chi0,ucvol)
!     ==================================================================
!     calculate chi0 for q!=0
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      type(dpparam) p
      real q(3)
      integer nr, nfft1, nfft2, nfft3, nfft1a
      complex chi0(p%npwmat,p%npwmat,p%nomega)
      integer igfft(p%npwvec,5,5,5)
      integer nbv
      real energy(p%nkibz,p%nbnds)
      real gwenergy(p%nkibz,p%nbnds)
      real occ(p%nkibz,p%nbnds)
      complex, target :: wfr(nr,p%nbnds,p%nkibz)
      real kbz(3,p%nkbz)
      integer ktab(p%nkbz), ktabr(nr,p%nkbz), ktabi(p%nkbz)
      real ucvol

      real kmq(3)
      integer g0(3)
      complex (kind(0.0d0)) :: den(p%nomega)
      complex (kind(0.0d0)) :: ediff, egwdiff
      double precision factocc
      real eckp, ockp

      complex, pointer :: wfnrvk(:), wfnrckp(:)
      complex rhotw(nr), rhotwg(p%npwmat), rhotwgs(p%npwmat)

      integer nrb
      integer ig, igp, ibv, ibc, i
      integer ik, ikp, ikibz, ikpibz
      integer ig01a, ig02a, ig03a
      integer io
      real weight
      logical wfnr_not_in_memory
      logical, external :: findkp

      print *, 'calculating chi0(q,omega,G,G")'
      print *

!     set up fft factor
      nrb = nfft1 * nfft2 * nfft3

!     zero chi0
      chi0(:,:,:) = (0.0,0.0)

!     weight (2 is for spin)
      weight = 2.0 / p%nkbz

!     loop over k-points in BZ
      do ik = 1, p%nkbz
        ikibz = ktab(ik)

!       identify k' and G0 where k' = k - q - G0
!       form k - q
        kmq(:) = kbz(:,ik) - q(:)
!       find k' = k - q - G0 and G0 also
        if(.not. findkp(p%nkbz,kbz,ikp,kmq,g0)) then
          print *, 'warning: cchi0 2: k" = k - q - G0 not found;'
          write(6,1927) ik, (kbz(i,ik),i=1,3), (kmq(i),i=1,3)
1927      format(' ik = ',i4,' k = ',3f12.6/' k - q = ',6x,3f12.6)
          cycle
        endif

!       g0 could also contain +-2
        ikpibz = ktab(ikp)
        ig01a = g0(1) + 3
        ig02a = g0(2) + 3
        ig03a = g0(3) + 3

!       loop over bands
        do ibc = p%lomo, p%nbnds
          eckp = gwenergy(ikpibz,ibc)
          ockp = occ(ikpibz,ibc)
          wfnrckp => wfr(:,ibc,ikpibz)

          do ibv = p%lomo, p%nbnds
            factocc = 0.5 * (ockp - occ(ikibz,ibv))
!           we continue the loop only if abs(factocc).ne.0. !!!
            if(abs(factocc) < 0.01) cycle

            egwdiff = eckp - gwenergy(ikibz,ibv)
            do io = 1, p%nomega
              den(io) = factocc/(p%omega(io)+egwdiff)
            enddo
            wfnrvk => wfr(:,ibv,ikibz)

!           form rho-twiddle(r) = u^*_ck'(r) * u_vk(r)
            call rho_tw_g(p%npwmat,nr,nrb,nfft1a,nfft1,nfft2,nfft3,
     &                    igfft(:,ig01a,ig02a,ig03a),
     &                    wfnrckp,ktabi(ikp),ktabr(:,ikp),
     &                    wfnrvk,ktabi(ik),ktabr(:,ik),
     &                    rhotwg)

!           chi0(G,G',io) = chi0(G,G',io) + (rhotwg(G) * rhotwg*(G')) * den(io)
            call assemblychi0(p%npwmat,p%nomega,chi0,rhotwg,den)

          enddo
        enddo
      enddo

      chi0(:,:,:) = weight * chi0(:,:,:) / ucvol

!     fill in the missing half of chi (G,G') using Hermitianness
!     Hermitianness is no longer valid for omega complex (not 0 not pure imag)
      do io = 1, p%nomega
        if(real(p%omega(io)) < 0.00001) then
          do igp = 1, p%npwmat
            do ig = 1, igp-1
              chi0(igp,ig,io) = conjg(chi0(ig,igp,io))
            enddo
          enddo
        endif
      enddo

!     now we have chi0(q,G,G',omega)

      return
      end


!     ==================================================================
      subroutine assemblychi0(npwmat,nomega,chi0,rhotwg,den)
!     ==================================================================
!     chi0(G,G',io) = chi0(G,G',io) + (rhotwg(G) * rhotwg*(G')) * den(io)
!     ------------------------------------------------------------------
      implicit none
      integer npwmat, nomega
      complex :: chi0(npwmat,npwmat,nomega)
      complex rhotwg(npwmat), rhotwgs(npwmat)
      complex (kind(0.0d0)) :: den(nomega)
      integer ig, igp, io, i
      complex d
!      complex m(npwmat,npwmat)
      integer npwmat2, i2
      logical, parameter :: VECTORIALIZED = .false.

      if(VECTORIALIZED) then
!       calculate rhotwg*(G)
        rhotwgs(:) = conjg(rhotwg(:))
!       npwmat2 = npwmat*npwmat
!       do i = 0, npwmat*npwmat*nomega-1
!         io  = i / npwmat2 + 1
!         ig  = (i - (i/npwmat2)*npwmat2) / npwmat + 1
!         igp = i - (i/npwmat2)*npwmat2 - 
!    &    ((i-(i/npwmat2)*npwmat2)/npwmat)*npwmat + 1
!         chi0(ig,igp,io) = chi0(ig,igp,io) +
!    &                      rhotwg(ig) * rhotwgs(igp) * den(io)
!       enddo
        do ig = 1, NPWMAT
          do igp = 1, NPWMAT
            do io = 1, nomega
              chi0(ig,igp,io) = chi0(ig,igp,io) +
     &                          rhotwg(ig) * rhotwgs(igp) * den(io)
            enddo
          enddo
        enddo
      else
!        m(:,:) = 0.0
!        call cher('U',npwmat,1.0,rhotwg,1,m,npwmat)
!        do ig = 1, npwmat
!          do igp = 1, ig-1
!            m(ig,igp) = conjg(m(igp,ig))
!          enddo
!        enddo
!        do io = 1, nomega
!          chi0(:,:,io) = chi0(:,:,io) + den(io) * m(:,:)
!        enddo
        do io = 1, nomega
!         d is single precision needed for cgerc
          d = den(io)
          call cgerc(npwmat,npwmat,d,rhotwg,1,rhotwg,1,
     &               chi0(:,:,io),npwmat)
        enddo
      endif
      return
      end


!     ==================================================================
      subroutine assemblywings(npwmat,nomega,lwing,uwing,rhotwg,den)
!     ==================================================================
!     chi0(G,G',io) = chi0(G,G',io) + (rhotwg(G) * rhotwg*(G')) * den(io)
!     ------------------------------------------------------------------
      implicit none
      integer npwmat, nomega
      complex :: lwing(npwmat,nomega), uwing(npwmat,nomega)
      complex rhotwg(npwmat), rhotwgs(npwmat)
      complex rhotwg1, rhotwgs1
      complex (kind(0.0d0)) :: den(nomega)
      integer io
!     calculate rhotwg*(G)
      rhotwgs(:) = conjg(rhotwg(:))
      rhotwg1 = rhotwg(1)
      rhotwgs1 = rhotwgs(1)
      do io = 1, nomega
        lwing(:,io) = lwing(:,io) +
     &                rhotwg(:) * rhotwgs1 * den(io)
        uwing(:,io) = uwing(:,io) +
     &                rhotwg1 * rhotwgs(:) * den(io)
      enddo
      return
      end





!     ==================================================================
      subroutine cVc(nq,q,b1,b2,b3,NPWMAT,gvec,qplusG,Vc)
!     ==================================================================
!     set up table of length of |q+G| for coulombian potential
!     and for coulombian potential Vc
!     ------------------------------------------------------------------
      use ggwdef
      implicit none
      integer nq
      real q(3,nq)
      real b1(3), b2(3), b3(3)
      integer NPWMAT
      integer gvec(3,*)
      real qplusG(NPWMAT,nq)
      real Vc(NPWMAT,nq)
      integer iq, ig
      real, external :: vlngth

      real gpq(3)

      print *, 'calculating coulombian potential'
      print *

      do iq = 1, nq
      if(all(abs(q(:,iq))<1.e-3)) then
        qplusG(1,iq) = vlngth(q(1,iq),b1,b2,b3)
        do ig = 2, NPWMAT
          gpq(1) = gvec(1,ig)
          gpq(2) = gvec(2,ig)
          gpq(3) = gvec(3,ig)
          qplusG(ig,iq) = vlngth(gpq,b1,b2,b3)
        enddo
      else
        do ig = 1, NPWMAT
          gpq(1) = gvec(1,ig) + q(1,iq)
          gpq(2) = gvec(2,ig) + q(2,iq)
          gpq(3) = gvec(3,ig) + q(3,iq)
          qplusG(ig,iq) = vlngth(gpq,b1,b2,b3)
        enddo
      endif
      enddo ! iq
      
      Vc(:,:) = 4 * PI / (qplusG(:,:)**2)

!     write (6,1038) iq,(ig,qplusG(ig),ig=1,NPWMAT)
!1038  format (' |G+q| for q-point ',I5,' (au):'/
!    &         (5(I5,':',F10.6)))

      return
      end
