! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

module ggwdef
implicit none

type crystal
  integer :: spgroup, pgroup                 ! space, point group
  double precision :: rprim(3,3)             ! dimensionless
  double precision :: gprim(3,3)             ! dimensionless
  double precision :: acell(3)               ! cell dimension in Bohr
  double precision :: rprimd(3,3)            ! in Bohr
  double precision :: gprimd(3,3)
  double precision :: ucvol, bzvol
  double precision :: gprimdm1(3,3)          ! transf from cc to rl
end type

type symmetry_operation
  integer :: r(3,3)                          ! rotation
  double precision :: t(3)                   ! translation
end type

type symmetry                                ! symmetry structure
  integer :: nsym
  type(symmetry_operation) :: sym(48)        ! maximum symmetries 48
end type

type pseudoatom
  double precision :: znucl
  double precision :: zion
  integer :: lmax
  integer :: lloc
  integer :: mmax
  integer :: pspso,pspdat,pspcod,pspxc
  character :: title(132)
end type

type chemistry
  integer :: ntype
  double precision, pointer :: znucl(:)
  integer :: natom
  integer, pointer :: type(:)
  double precision, pointer :: xred(:,:)     ! atom reduced coordinates
  integer :: npsa
  type(pseudoatom), pointer :: psa(:)
  integer :: nel        
end type

type mesh
  integer ng
  integer nr
  integer ngg(3)
  integer nrr(3)
end type

type transposed_space
  integer ng
  integer nsh
  integer nk
  integer, pointer :: g(:,:)
  integer, pointer :: sh(:)
  double precision, pointer :: k(:,:)        ! IBZ
end type  

type electronic_structure
  integer nb
  integer nk
  integer ng
  integer nr
  double precision, pointer :: en(:,:)       ! en(n,k) Bloch order
  double precision, pointer :: oc(:,:)       ! same
  double precision, pointer :: wfg(:,:,:)    ! wfg(g,n,k) order
  double precision, pointer :: wfr(:,:,:)    ! wfr(r,n,k) order
  double precision :: fermie
  double precision :: etotal, residm, ecut_eff
  integer :: xc
end type

type kb_potential
  integer :: ntype
  integer :: natom
  integer :: mpsang
  integer :: ng
  integer :: nk
  real, pointer :: vkbsign(:,:), vkb(:,:,:,:), vkbd(:,:,:,:)
  complex, pointer :: gradvnl(:,:,:,:)
  complex, pointer :: fnl(:,:,:,:), fnld(:,:,:,:,:)
end type        


!      type rlv
!        integer rlu(3)                          ! gvec
!        integer jgfft                           ! igfft(G0=0)
!        integer igfft(-3:3,-3:3,-3:3)           ! igfft
!        type(rlv), pointer :: grottb(:,:)       ! grottb(ninv,nop)
!      endtype

!      type ibzpoint
!        real rlu(3)                             ! kibz, qibz
!        real w                                  ! weight
!        real, pointer :: en(:)                  ! energy
!      endtype
      
!      type bzpoint
!        real rlu(3)                             ! kbz, qbz
!        type(ibzpoint), pointer :: ibz          ! ktab, qtab
!        type(symmop), pointer :: o              ! ktabo, qtabo
!        logical :: i                            ! ktabi, qtabi
!      endtype

!      type dlv
!        integer, pointer :: irottb(:)           ! irottb(nop)
!      endtype
!     ktabr(nr,nkbz) -> ktabr(:,kbz) = irottb(:,iop)
      

      type lattstruct                           ! deprecated: use crystal
        integer lattype
        real acell(3)                           ! cell dimension in Bohr
        real rprim(3,3)                         ! dimensionless
        real r(3,3)                             ! in Bohr
        real a1(3), a2(3), a3(3)
        real b1(3), b2(3), b3(3)
        real ucvol, bzvol
        real brl2cc(3,3)
        real bcc2rl(3,3)
      endtype
      
      type transition
!       temporary an integer but bzpoint in the future
        integer k
        integer v
        integer c
        real en
      endtype

real, parameter :: HARTREE = 27.2116
real, parameter :: PI = 3.14159265358979323846
double precision, parameter :: zero = 0.0d0

end module
