! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

! interface to the FFTW library

integer function nfft(n)
  integer m
  integer, external :: allowed_fftsize
  m=allowed_fftsize(n)
  nfft=m
end
integer function allowed_fftsize(n)
  ! calculate the size of the smallest allowed FFT size that will encompass n
  ! in FFTW all is allowed
  integer n
  allowed_fftsize = n
  return
end


subroutine c3dfft(c,id,ie,if,nl,nm,nn,isig)
  ! computes the forward fourier transform of a complex
  ! function c. the fourier transform is
  ! returned in c in output (the input c is overwritten)
  ! *****     single precision fourier transform
  !           isig <0  inverse
  !           isig >0  direct
  implicit none
  include "/usr/local/include/fftw3.f"
  integer id, ie, if
  integer nl, nm, nn
  integer isig
  complex c(id,ie,if)
  integer, save :: nlo=0, nmo=0, nno=0, isigo=1
  complex (kind(0.0d0)), save,  allocatable :: b(:,:,:)
  integer (8), save :: plan

  if(nl.ne.nlo .or. nm.ne.nmo .or. nn.ne.nno .or. isig.ne.isigo) then
    ! initialize
    if(nlo/=0 .or. nmo/=0 .or. nno/=0) then
      call dfftw_destroy_plan(plan)
     deallocate(b)
    endif
    allocate(b(nl,nm,nn))
    if(isig<0) then
      call dfftw_plan_dft_3d(plan,nl,nm,nn,b,b,FFTW_BACKWARD,FFTW_MEASURE)
    else 
      call dfftw_plan_dft_3d(plan,nl,nm,nn,b,b,FFTW_FORWARD,FFTW_MEASURE)
    endif
    nlo = nl ; nmo = nm ; nno = nn; isigo = isig
  endif
  b(:,:,:) = c(1:nl,1:nm,1:nn)
  call dfftw_execute(plan)
  c(1:nl,1:nm,1:nn) = b(:,:,:)
  return
end

