! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

c     ==================================================================
c     interface to the NAG library
c     ==================================================================


c     ==================================================================
      subroutine chevc(n,c,ldc,w,z,ldz)
c     ==================================================================
c     Compute the eigenvalues and the eigenvectors
c     of a complex Hermitian matrix
c     ------------------------------------------------------------------

      integer n, ldc, ldz
      complex c(ldc,n)
      real w(n)
      complex z(ldz,n)

      integer ifail
      real*8, allocatable :: ar(:,:), ai(:,:)
      real*8, allocatable :: r(:)
      real*8, allocatable :: wk1(:), wk2(:), wk3(:)

      allocate(ar(n,n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ai(n,n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(r(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk1(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk2(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk3(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      do j = 1, n
        do i = 1, n
          ar(i,j) = dble(real(c(i,j)))
          ai(i,j) = dble(aimag(c(i,j)))
        enddo
        ai(j,j) = 0.d0
      enddo

      ifail = 0
c     call f02awf(ar,n,ai,n,n,r,wk1,wk2,wk3,ifail)

      call f02axf(ar,n,ai,n,n,r,ar,n,ai,n,wk1,wk2,wk3,ifail)
      if(ifail.ne.0) stop '** ERROR chevc'

      do j = 1, n
        do i = 1, n
          z(i,j) = cmplx(sngl(ar(i,j)),sngl(ai(i,j)))
        enddo
        w(j) = sngl(r(j))
      enddo

      deallocate(ar)
      deallocate(ai)
      deallocate(r)
      end


c     ==================================================================
      subroutine zhevc(n,c,ldc,w,z,ldz)
c     ==================================================================
c     Compute the eigenvalues and the eigenvectors
c     of a complex*16 Hermitian matrix
c     ------------------------------------------------------------------

      integer n, ldc, ldz
      complex*16 c(ldc,n)
      real*8 w(n)
      complex*16 z(ldz,n)

      integer ifail
      real*8, allocatable :: ar(:,:), ai(:,:)
      real*8, allocatable :: r(:)
      real*8, allocatable :: wk1(:), wk2(:), wk3(:)

      allocate(ar(n,n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ai(n,n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(r(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk1(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk2(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wk3(n),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      do j = 1, n
        do i = 1, n
          ar(i,j) = real(c(i,j))
          ai(i,j) = aimag(c(i,j))
        enddo
        ai(j,j) = 0.d0
      enddo

      ifail = 0
c     call f02awf(ar,n,ai,n,n,r,wk1,wk2,wk3,ifail)

      call f02axf(ar,n,ai,n,n,r,ar,n,ai,n,wk1,wk2,wk3,ifail)
      if(ifail.ne.0) stop '** ERROR zhevc'

      do j = 1, n
        do i = 1, n
          z(i,j) = cmplx(ar(i,j),ai(i,j))
        enddo
        w(j) = r(j)
      enddo

      deallocate(ar)
      deallocate(ai)
      deallocate(r)
      end



!     ==================================================================
!     Fourier transforms
!     ==================================================================

!     ==================================================================
      integer function allowed_fftsize(n)
!     ==================================================================
!     calculate the size of the smallest allowed FFT size that
!     will encompass n
!     this routine is written for the ESSL routine scft, in which the
!     allowable sizes of FFT are those whose prime factors are
!       n <= 37748736
!       n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!     ------------------------------------------------------------------
      implicit none
      integer n
      integer, parameter :: nn = 56
      integer nallwd(nn)
      data nallwd/
     &   2,   4,   6,   8,  10,  12,  14,  16,  18,  20,
     &  22,  24,  28,  30,  32,  36,  40,  42,  44,  48,
     &  56,  60,  64,  66,  70,  72,  80,  84,  88,  90,
     &  96, 110, 112, 120, 126, 128, 132, 140, 144, 154,
     & 160, 168, 176, 180, 192, 198, 210, 220, 224, 240,
     & 252, 256, 264, 280, 288, 308/
      integer i

      if(n > nallwd(nn) .or. n < 2) then
        print *, '**ERROR allowed_fftsize'
        stop 'error allowed fft_size'
      endif
      do i = nn, 1, -1
        if(nallwd(i) >= n) allowed_fftsize = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine nfft(m,n)
!     ==================================================================
!     calculate N, the size of the smallest allowed FFT that will encompass
!     the 2M+1 points -M..+M.  (Ideally, N=2M+1, but this may not be
!     an allowable FFT.)
!     this routine is written for the ESSL routine scft, in which the
!     allowable sizes of FFT are those whose prime factors are
!       n <= 37748736
!       n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!     ------------------------------------------------------------------
      implicit none
      integer n, m
      integer, parameter :: nn = 56
      integer nallwd(nn)
      data nallwd/
     &   2,   4,   6,   8,  10,  12,  14,  16,  18,  20,
     &  22,  24,  28,  30,  32,  36,  40,  42,  44,  48,
     &  56,  60,  64,  66,  70,  72,  80,  84,  88,  90,
     &  96, 110, 112, 120, 126, 128, 132, 140, 144, 154,
     & 160, 168, 176, 180, 192, 198, 210, 220, 224, 240,
     & 252, 256, 264, 280, 288, 308/
      integer i, nbest
      
      nbest = 2 * m + 1
      if(nbest .gt. nallwd(nn)) then
        write(*,*) '** ERROR nfft 1: '
        stop
      endif
      if(nbest .lt. 2) then
        write(*,*) '** ERROR nfft 2: '
        stop
      endif
      do i = nn, 1, -1
        if(nallwd(i) .ge. nbest) n = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine n1dfft(n1,n1d)
!     ==================================================================
!     set the first dimension n1d of the FFT 3d arrays to be n1
!     ------------------------------------------------------------------
      implicit none
      integer n1, n1d
      n1d = n1
      return
      end      

      
      
c     ==================================================================
      subroutine cfft(n,x,y,isign)
c     ==================================================================
c     compute Fourier transform of a complex periodic sequence
c     the array x and x may be the same
c     ------------------------------------------------------------------
      integer n
      complex x(n)
      complex y(n)
      integer isign
      
      integer ifail
      real work(n)
      real r(n)
      real i(n)
      integer j
      
      ifail = 0
      
      do j = 1, n
        r(j) = real(x(j))
        i(j) = aimag(x(j))
      enddo
      
      call c06fcf(r,i,n,work,ifail)
      
      do j = 1, n
        y(j) = cmplx( r(j) , i(j) )
      enddo
         
      return
      end

      
c     ==================================================================
      subroutine c3dfft(n1,n2,n3,a,lda,mda,b,ldb,mdb,isign)
c     ==================================================================
c     compute Fourier transform of a complex periodic three-dimensional array
c     the matrices a and a may be the same
c     ------------------------------------------------------------------
      integer n1, n2, n3
      complex a(lda,mda,*)
      complex b(ldb,mdb,*)
      integer isign
      
      integer ifail
      character*1, save :: init
      real, allocatable, save :: trign1(:), trign2(:), trign3(:)
      real, allocatable :: x(:,:,:), y(:,:,:)
      real, allocatable :: work(:)
      logical, save :: linit = .false.
      integer istat
      
      allocate(work(2*n1*n2*n3))
      allocate(x(n1,n2,n3))
      allocate(y(n1,n2,n3))
      
      if(.not.linit) then
        init = 'I'
        ifail = 0
        allocate(trign1(2*n1),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(trign2(2*n2),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(trign3(2*n3),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        
        call c06fxf(n1,n2,n3,x,y,init,trign1,trign2,trign3,work,ifail)
        
        init = 'S'
        linit = .true.
      endif
      
      ifail = 0    
      
      do i = 1, n1
        do j = 1, n2
          do k = 1, n3
            x(i,j,k) = real(a(i,j,k))
            y(i,j,k) = aimag(a(i,j,k))
          enddo
        enddo
      enddo
      
      call c06fxf(n1,n2,n3,x,y,init,trign1,trign2,trign3,work,ifail)
      
      do i = 1, n1
        do j = 1, n2
          do k = 1, n3
            b(i,j,k) = cmplx( x(i,j,k) , y(i,j,k) )
          enddo
        enddo
      enddo
      
      deallocate(x,y,work)
      
      return
      end

