! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
      subroutine rddpin(inputfile,il,p)
!     ==================================================================
!     read all the parameters of the dp calculation
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      integer il
      character (len=il) :: inputfile
      type(dpparam) p
      
      integer, external :: read_int, read_string, read_float, read_file
      integer, external :: read_float_array
!      character (len=12) :: ttype
      integer i, iu
      integer te, tp
      
      integer, parameter :: nctype = 20
      integer ctype(nctype), maxctype, itype
      character (len=12) sctype(nctype)

      if(il==0) then
        print *, 'reading parameters from stdin'
        print *, '(type CTRL^D when finished)'
        if(read_file('stdin',5)/=1) stop 'error'
      else
        print *, 'reading parameters from file ', inputfile
        if(read_file(inputfile,il) /= 1) stop 'error in inputfile'
      endif
      print *
      
      p%ns = 1 ! spin unpolarized

!     read information about the dielectric matrix to be considered
!     test-particle or test-electron
      te = read_string('testelectron',12)
      tp = read_string('testparticle',12)
      if(tp>=te) then
        p%TESTPARTICLE = .true.
        p%TESTELECTRON = .false.
        p%ttype = 'testparticle'
      else
        p%TESTELECTRON = .true.
        p%TESTPARTICLE = .false.
        p%ttype = 'testelectron'
      endif

!     read information about calculation type
!     rpa or lda or etc.
      sctype(1) = 'rpa'
       ctype(1) = read_string('rpa',3)
      sctype(2) = 'lda'
       ctype(2) = read_string('lda',3)
      sctype(3) = 'nlda'
       ctype(3) = read_string('nlda',4)
      sctype(4) = 'so'
       ctype(4) = read_string('so',2)
      sctype(5) = 'gw'
       ctype(5) = read_string('gw',2)

      sctype(6)  = 'rpa+a'
       ctype(6)  = read_string('rpa+a',5)
      sctype(7)  = 'rpa+eps'
       ctype(7)  = read_string('rpa+eps',7)
      sctype(8)  = 'rpa+epsdiago'
       ctype(8)  = read_string('rpa+epsdiago',12)
      sctype(9)  = 'rpa+vbar'
       ctype(9)  = read_string('rpa+vbar',8)
      sctype(10) = 'rpa+cutoff'
       ctype(10) = read_string('rpa+cutoff',10)

      sctype(11) = 'so+a'
       ctype(11) = read_string('so+a',4)
      sctype(12) = 'so+eps'
       ctype(12) = read_string('so+eps',6)
      sctype(13) = 'so+epsdiago'
       ctype(13) = read_string('so+epsdiago',11)
      sctype(14) = 'so+vbar'
       ctype(14) = read_string('so+vbar',7)
      sctype(15) = 'so+cutoff'
       ctype(15) = read_string('so+cutoff',9)

      sctype(16) = 'gw+a'
       ctype(16) = read_string('gw+a',4)
      sctype(17) = 'gw+eps'
       ctype(17) = read_string('gw+eps',6)
      sctype(18) = 'gw+epsdiago'
       ctype(18) = read_string('gw+epsdiago',11)
      sctype(19) = 'gw+vbar'
       ctype(19) = read_string('gw+vbar',7)
      sctype(20) = 'gw+cutoff'
       ctype(20) = read_string('gw+cutoff',9)

      maxctype = 0
      itype = 1 ! default is RPA
      do i = 1, nctype
        if(ctype(i) > maxctype) then
          maxctype = ctype(i)
          itype = i
        endif
      enddo
      p%type = sctype(itype)

!     read fundamental parameters
      if(read_int('npwmat',6,p%npwmat)==0) p%npwmat = 1
      if(read_int('npwwfn',6,p%npwwfn)==0) p%npwwfn = -1
      if(read_int('nbands',6,p%nbnds)==0) p%nbnds = -1
      if(read_int('lomo',4,p%lomo)==0) p%lomo = 1
      if(read_int('nkibz',5,p%nkibz)==0) p%nkibz = -1
      if(read_int('nel',3,p%nel)==0) p%nel = -1
      if(read_int('nsym',4,p%nop)==0) p%nop = -1
      p%inv = .true.
      p%ninv = 2
      if(read_string('noinv',5)>0) p%inv = .false.
      if(.not.p%inv) p%ninv = 1


      if(read_float_array('q',1,p%qpg,3)==0) p%qpg(:) = 0.0
      if(read_float('omegai',6,p%omegai)==0) p%omegai = 0.0
      if(read_float('omegae',6,p%omegae)==0) p%omegae = 30.0
      if(read_float('domega',6,p%domega)==0) p%domega = 0.1
      if(read_float('broad',5,p%broad)==0) p%broad = p%domega

      if(read_float('soenergy',8,p%soenergy)==0) p%soenergy = 0.0
      if(read_float('alpha',5,p%alphadiv)==0) p%alphadiv = 0.0
      if(read_float('rcut',4,p%rcut)==0) p%rcut = 3.0
      if(read_float('beta',4,p%beta)==0) p%beta = 1.0

      if(read_int('alias',5,p%alias)==0) p%alias = 2
      if(read_string('aliasfree',9)>0) p%alias = 2

!     set flags of the calculation depending on input
      p%RPA = .false.
      p%LDA = .false.
      p%NLDA = .false.
      p%GW = .false.
      p%SO = .false.
      p%FXCALPHA = .false.
      p%FXCEPS = .false.
      p%FXCEPSDIAGO = .false.
      p%CUTOFF=.false.
      p%FXCVBAR=.false.
      if(p%type.eq.'LDA'.or.p%type.eq.'lda') then
        p%LDA = .true.
      else if(p%type.eq.'GW'.or.p%type.eq.'gw') then
        p%GW = .true.
      else if(p%type.eq.'GW+A'.or.p%type.eq.'gw+a') then
        p%GW = .true.
        p%FXCALPHA = .true.
      else if(p%type.eq.'GW+EPS'.or.p%type.eq.'gw+eps') then
        p%GW = .true.
        p%FXCEPS = .true.
      else if(p%type.eq.'GW+EPSDIAGO'.or.p%type.eq.
     &        'gw+epsdiago') then
        p%GW = .true.
        p%FXCEPSDIAGO = .true.
      else if(p%type.eq.'GW+VBAR'.or.p%type.eq.'gw+vbar') then
        p%GW = .true.
        p%FXCVBAR = .true.
      else if(p%type.eq.'GW+CUTOFF'.or.p%type.eq.'gw+cutoff') then
        p%GW = .true.
        p%CUTOFF = .true.
      else if(p%type.eq.'SO'.or.p%type.eq.'so'.or.
     &        p%type.eq.'K'.or.p%type.eq.'k') then
        p%SO = .true.
      else if(p%type.eq.'SO+A'.or.p%type.eq.'so+a') then
        p%SO = .true.
        p%FXCALPHA = .true.
      else if(p%type.eq.'SO+EPS'.or.p%type.eq.'so+eps') then
        p%SO = .true.
        p%FXCEPS = .true.
      else if(p%type.eq.'SO+EPSDIAGO'.or.p%type.eq.
     &        'so+epsdiago') then
        p%SO = .true.
        p%FXCEPSDIAGO = .true.
      else if(p%type.eq.'SO+VBAR'.or.p%type.eq.'so+vbar') then
        p%SO = .true.
        p%FXCVBAR = .true.
      else if(p%type.eq.'SO+CUTOFF'.or.p%type.eq.'so+cutoff') then
        p%SO = .true.
        p%CUTOFF = .true.
      else if(p%type.eq.'NLDA'.or.p%type.eq.'nlda') then
        p%NLDA = .true.
      else if(p%type.eq.'RPA+A'.or.p%type.eq.'rpa+a') then
        p%RPA = .true.
        p%FXCALPHA = .true.
      else if(p%type.eq.'RPA+EPS'.or.p%type.eq.'rpa+eps') then
        p%RPA = .true.
        p%FXCEPS = .true.
      else if(p%type.eq.'RPA+EPSDIAGO'.or.p%type.eq.
     &        'rpa+epsdiago') then
        p%RPA = .true.
        p%FXCEPSDIAGO = .true.
      else if(p%type.eq.'RPA+VBAR'.or.p%type.eq.'rpa+vbar') then
        p%RPA = .true.
        p%FXCVBAR = .true.
      else if(p%type.eq.'RPA+CUTOFF'.or.p%type.eq.'rpa+cutoff') then
        p%RPA = .true.
        p%CUTOFF = .true.
      else
        p%RPA = .true.
      endif

      p%NOFXC = .true.
      if(p%LDA .or. p%TESTELECTRON) p%NOFXC = .false.
      if(p%FXCALPHA .or. p%FXCEPS .or. p%FXCEPSDIAGO
     &   .or. p%CUTOFF .or. p%FXCVBAR) p%NOFXC = .false.
 
      if(p%TESTELECTRON .and. .not.p%LDA) stop 'TestElectron'


      p%QZERO = .false.
      if(all(abs(p%qpg(:))<1.e-2)) p%QZERO = .true.
      p%nq = 1
      p%nomega = (p%omegae - p%omegai) / p%domega + 0.5 + 1

!     conversion to Hartree
      p%omegai = p%omegai / HARTREE
      p%omegae = p%omegae / HARTREE
      p%domega = p%domega / HARTREE
      p%broad  = p%broad  / HARTREE
      p%soenergy = p%soenergy / HARTREE
      
      return
      end


!     ==================================================================
      subroutine print_parameters(p)
!     ==================================================================
!     write all the parameters of the dp calculation
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      type(dpparam) p
      integer i

      write(*,*) 'Epsilon ', p%ttype
      write(*,*) p%type

!     print parameters
      write(6,1000) p%npwmat, p%npwwfn, p%nbnds, p%lomo,
     &              p%nel, p%nkibz, p%nop, p%inv
1000  format(' fundamental parameters:'/
     & ' dimension of the matrices,     npwmat = ',i5/
     & ' cut-off on the wavefunctions,  npwwfn = ',i5/
     & ' number of bands included,      nbands = ',i5/
     & ' lowest occupied band included, lomo   = ',i5/
     & ' number of electrons            nel    = ',i5/
     & ' max number of k-points in IBZ, nkibz  = ',i5/
     & ' number of symmetry on k-set,   nsym   = ',i5/
     & ' inversion on k-set,            inv    = ',l5)

      write(6,2000) (p%qpg(i),i=1,3)
2000  format(
     & ' q-point [reciprocal lattice units]    = ',(3f12.6))

      write(*,3000) p%omegai*HARTREE, p%omegae*HARTREE,
     &              p%domega*HARTREE, p%broad*HARTREE
3000  format(
     & ' minimum energy [eV]            Emin   = ',f6.2/
     & ' maximum energy [eV]            Emax   = ',f6.2/
     & ' energy step    [eV]            dE     = ',f6.2/
     & ' lorentzian broadening [eV]     eta    = ',f6.2)
      

      if(p%SO)  write(*,'(" SO ",f5.2," [eV]")') p%soenergy*HARTREE
      if(p%FXCALPHA) write(*,'(" alpha ",f6.2)') p%alphadiv
      print *

      return
      end


!     ==================================================================
      subroutine wreps(p,tpula,epslf,epsnlf,dsffact)
!     ==================================================================
!     write all the files of the dp calculation
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      type(dpparam) p
      real tpula
      real omegaev
      real domega
      real domev
      complex epslf(p%nomega,p%nq+2), epsnlf(p%nomega,p%nq+2)
      real epsmlf0(8), epsmnlf0(8)
      real dsffact
      real dsflf(p%nomega,p%nq+2), dsfnlf(p%nomega,p%nq+2)
      real eellf(p%nomega,p%nq+2), eelnlf(p%nomega,p%nq+2)
      real nlf(p%nomega,p%nq+2), nnlf(p%nomega,p%nq+2)
      real klf(p%nomega,p%nq+2), knlf(p%nomega,p%nq+2)
      real rfllf(p%nomega,p%nq+2), rflnlf(p%nomega,p%nq+2)
      real ldrfllf(p%nomega,p%nq+2), ldrflnlf(p%nomega,p%nq+2)
      integer io, iq, nnq
      character (len=10) filename

      domega = real(p%omega(p%nomega) - p%omega(1)) / (p%nomega - 1)
      domev = domega * HARTREE

      if(p%nq == 1) then
        nnq = 1
      else
        nnq = 8        
!       calculating reciprocal lattice and cartesian averages
        do io = 1, p%nomega
          epslf(io,7) = (epslf(io,1)+epslf(io,2)+epslf(io,3)) / 3.0
          epsnlf(io,7) = (epsnlf(io,1)+epsnlf(io,2)+epsnlf(io,3)) / 3.0
          epslf(io,8) = (epslf(io,4)+epslf(io,5)+epslf(io,6)) / 3.0
          epsnlf(io,8) = (epsnlf(io,4)+epsnlf(io,5)+epsnlf(io,6)) / 3.0
        enddo
      endif

      if(p%QZERO .and. p%ig == 1) then
        if(p%omegai < 0.1/HARTREE) then
          do iq = 1, nnq
            epsmlf0(iq) = real(epslf(1,iq))
            epsmnlf0(iq) = real(epsnlf(1,iq))
          enddo
          do iq = 1, p%nq
            print *, 'q-direction ', iq
            write(6,1975) epsmlf0(iq), epsmnlf0(iq)
1975        format(' dielectric constant epsilon0 = ',f8.4,/,
     &            ' dielectric constant without local fields = ',f8.4)
            if(abs(aimag(p%omega(1))) > 0.001)
     &        write(6,1980) aimag(p%omega(1))*HARTREE
1980        format(' warning: broadening = ',f5.2,' [eV]')
            print *
          enddo
          if(nnq>1) then
            print *, 'q average of reciprocal coordinates directions'
            write(6,1975) epsmlf0(7), epsmnlf0(7)
            print *
            print *, 'q average of cartesian directions'
            write(6,1975) epsmlf0(8), epsmnlf0(8)
            print *
          endif
        endif
      endif
      
      
      do iq = 1, nnq
       do io = 1, p%nomega
        eellf(io,iq)  = aimag(-(1.0,0.0)/epslf(io,iq))
        eelnlf(io,iq) = aimag(-(1.0,0.0)/epsnlf(io,iq))
!       the dynamic structure factor is
!       S(q+G) = - (q+G)^2 / (4 PI^2 e^2 n)    * Im(e^-1_GG(q,omega))
        dsflf(io,iq)  = dsffact * eellf(io,iq)
        dsfnlf(io,iq) = dsffact * eelnlf(io,iq)
        nlf(io,iq)    = real(sqrt(epslf(io,iq)))
        nnlf(io,iq)   = real(sqrt(epsnlf(io,iq)))
        klf(io,iq)    = aimag(sqrt(epslf(io,iq)))
        knlf(io,iq)   = aimag(sqrt(epsnlf(io,iq)))
        rfllf(io,iq)  = ((nlf(io,iq)-1)**2 + klf(io,iq)**2) / 
     &               ((nlf(io,iq)+1)**2 + klf(io,iq)**2)
        rflnlf(io,iq) = ((nnlf(io,iq)-1)**2 + knlf(io,iq)**2) / 
     &               ((nnlf(io,iq)+1)**2 + knlf(io,iq)**2)
        if(io > 1) then
          ldrfllf(io,iq)  = (1-rfllf(io-1,iq)/rfllf(io,iq)) / domev
          ldrflnlf(io,iq) = (1-rflnlf(io-1,iq)/rflnlf(io,iq)) / domev
        else
          ldrfllf(io,iq)  = 0.0
          ldrflnlf(io,iq) = 0.0
        endif
       enddo
      enddo

      if(p%nq==1) then
        iq = 1
        open(unit=22,file='outlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# with local fields")')
        call wrtitle(22,p,tpula,dsffact)
        call wrq(22,p,tpula,1)
        write(22,1234)
 1234   format("# omega       eps_1       eps_2")
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,2e12.4)') omegaev, epslf(io,iq)
        enddo
        close(22)
        if(p%NOFXC) then
        open(unit=22,file='outnlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# without local fields")')
        call wrtitle(22,p,tpula,dsffact)
        call wrq(22,p,tpula,1)
        write(22,1234)
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,2e12.4)') omegaev, epsnlf(io,iq)
        enddo
        close(22)
        endif
      else
        iq = 7
        open(unit=22,file='ourlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# with local fields")')
        call wrtitle(22,p,tpula,dsffact)
        call wrq(22,p,tpula,iq)
        write(22,1234)
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,2e12.4)') omegaev, epslf(io,iq)
        enddo
        close(22)
        if(p%NOFXC) then
        open(unit=22,file='ournlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# without local fields")')
        call wrtitle(22,p,tpula,dsffact)
        call wrq(22,p,tpula,iq)
        write(22,1234)
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,2e12.4)') omegaev, epsnlf(io,iq)
        enddo
        close(22)
        endif

        open(unit=22,file='outlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# with local fields")')
        call wrtitle(22,p,tpula,dsffact)
        write(22,'("# 8 polarization:")')
        call wrq(22,p,tpula,7)
        call wrq(22,p,tpula,8)
        do iq = 1, p%nq
          call wrq(22,p,tpula,iq)
        enddo
        write(22,1235)
 1235   format("# omega       eps_1       eps_2  for r c 1 2 3 x y z")
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,16e12.4)') omegaev, epslf(io,7), epslf(io,8),
     &      epslf(io,1), epslf(io,2), epslf(io,3),
     &      epslf(io,4), epslf(io,5), epslf(io,6)
        enddo
        close(22)
        if(p%NOFXC) then
        open(unit=22,file='outnlf.eps',form='formatted')
        write(22,'("# macroscopic dielectric function")')
        write(22,'("# without local fields")')
        call wrtitle(22,p,tpula,dsffact)
        call wrq(22,p,tpula,7)
        call wrq(22,p,tpula,8)
        do iq = 1, p%nq
          call wrq(22,p,tpula,iq)
        enddo
        write(22,1235)
        do io = 1, p%nomega
          omegaev = real(p%omega(io))*HARTREE
          write(22,'(f7.3,16e12.4)') omegaev, epsnlf(io,7),epsnlf(io,8),
     &      epsnlf(io,1), epsnlf(io,2), epsnlf(io,3),
     &      epsnlf(io,4), epsnlf(io,5), epsnlf(io,6)
        enddo
        close(22)
        endif
        
      endif
      

      return
      end


!     ==================================================================
      subroutine wrtitle(unit,p,tpula,dsffact)
!     ==================================================================
!     write title for the files of the dp calculation
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      integer unit
      type(dpparam) p
      real tpula
      real dsffact
      
      write(unit,'("# ",a12,2x,a20)') p%ttype,p%type
      if(p%SO) write(unit,'("# so = ",f5.2," [eV]")') p%soenergy*HARTREE
      if(p%FXCALPHA) write(unit,'("# alpha = ",f5.2)') p%alphadiv
      
      write(unit,300) p%npwmat,p%npwwfn,p%nbnds,p%lomo,p%nkibz,p%nkbz
300   format('# npwmat =',i4,'  npwwfn =',i4,'  nbands =',i4,
     &       '  lomo =',i4,'  nkibz =',i4,'  nkbz =',i4)
      write(unit,'("# energy range ",2f6.2," [eV]")')
     &  p%omegai*HARTREE, p%omegae*HARTREE
      write(unit,1000) p%broad*HARTREE
1000  format('# lorentzian broadening = ',f6.4,' [eV]')

      write(unit,'("# dsf factor ",f12.6," [a.u.]")') dsffact

      write(unit,700) p%g(:)
700   format('# G = (',i3,',',i3,',',i3,') [r.l.u.]')
      write(unit,800) p%gcc(:)
800   format('# G = (',f9.6,',',f9.6,',',f9.6,') [c.c.]')
      write(unit,900) p%gcc2pioa(:), tpula
900   format('# G = (',f9.6,',',f9.6,',',f9.6,') ',
     &       '[c.c. units of 2pi/a = ',f9.6,' a.u.]')

      return
      end


!     ==================================================================
      subroutine wrq(unit,p,tpula,iq)
!     ==================================================================
!     write q for the files of the dp calculation
!     ------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      integer unit
      type(dpparam) p
      real tpula
      integer iq

      if(iq <= 6) then
        write(unit,400) p%q(:,iq)
400   format('# q = (',f9.6,',',f9.6,',',f9.6,') [r.l.u.]')
        write(unit,500) p%qcc(:,iq)
500   format('# q = (',f9.6,',',f9.6,',',f9.6,') [c.c.]')
        write(unit,600) p%qcc2pioa(:,iq), tpula
600   format('# q = (',f9.6,',',f9.6,',',f9.6,') ',
     &       '[c.c. units of 2pi/a = ',f9.6,' a.u.]')
      else if(iq == 7) then
        write(unit,410)
410     format('# q -> 0 ',
     &    'average over the three reciprocal lattice directions')
      else if(iq == 8) then
        write(unit,420)
420     format('# q -> 0 ',
     &    'average over the three cartesian directions')
      endif

      return
      end
     
