! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!======================================================================
      subroutine rdgw(gwfile,gl,
     &  nkibz,nkbz,nbnds,lomo,kbz,ktab,gwen,gwsh,nkibzm)
!======================================================================
!     read GW-energy-parameters
!
!     input:
!       gwfile
!----------------------------------------------------------------------
      integer gl
      character (len=gl) :: gwfile
      integer nkibzm
      integer nkibz, nkbz, nbnds, lomo
      integer ktab(nkbz)
      real kbz(3,nkbz)
      real gwen(nkibzm,nbnds)
      real gwsh(nkibzm,nbnds)
      
      real en
      real gw
      
      integer ik, ikibz
      integer ibcheck, ibcheckok
      
      integer nk
      real k(3), k2bz(3)
      integer g1, g2, g3
      real HARTREE
      parameter(HARTREE=27.2116)

      write(*,*) 'reading GW energies and GW corrections'
      
      gwsh(:,:) = 0.0
      gwen(:,:) = 0.0

      ibcheckok = 0
      do ib = lomo, nbnds
        ibcheckok = ibcheckok + ib
      enddo

c     opening file
      open(13,file=gwfile,status='unknown')

c     reading number of k-points
      read(13,*) nk
      write(*,*) 'number of k-points found ', nk
      
      if(nk.lt.nkibz) then
        write(*,*) 'warning: the number of k-points found is less than '
     &             , nkibz
      endif


      do j = 1, nk

        read(13,*) (k(i),i=1,3)
 
c       determine the index number in bz
        ik = 0
        do jk = 1, nkbz
          if(all(abs(k-kbz(1:3,jk)) < 1.e-4)) ik = jk
        enddo
        if(ik==0) then
          write(*,1023) (k(i),i=1,3)
1023      format ('trying to find k =',3f14.6,
     &            ' in other BZ zones')
          do g1 = -1, 1
          do g2 = -1, 1
          do g3 = -1, 1
            k2bz(1) = k(1) + g1
            k2bz(2) = k(2) + g2
            k2bz(3) = k(3) + g3
            do jk = 1, nkbz
              if(all(abs(k2bz-kbz(1:3,jk)) < 1.e-4)) ik = jk
            enddo
          enddo
          enddo
          enddo
        endif
        if(ik .eq. 0) then
          write(*,1071) (k(i),i=1,3)
1071      format ('**ERROR, rdgw: k-point ',3f14.6,
     &            ' is not in set of kBZ!')
          stop '**ERROR, rdgw: k-point not in the set'
        endif

        ikibz = ktab(ik)
        write(*,100) ik, ikibz, kbz(:,ik)
100     format('  k-point, BZ number ',i3,'  IBZ number ',i3,
     &         '  r.l.c. ',3f6.3)

        read(13,*) nb
        write(*,*) '  number of bands found ', nb

        if(nb < (nbnds-lomo+1)) then
          write(*,*) 'warning: the number of bands found is less than '
     &             , nbnds-lomo+1
        endif
        
        write(*,150)
150     format(3x,'band',3x,'gw energy',4x,'gw shift')
        ibcheck = 0  
        do jb = 1, nb
          read(13,*) ib, en, gw
          en = en / HARTREE
          gw = gw / HARTREE
          if(ib <= nbnds) then
            gwsh(ikibz,ib) = gw
            gwen(ikibz,ib) = en
            write(*,200) ib, gwen(ikibz,ib)*HARTREE,
     &                       gwsh(ikibz,ib)*HARTREE
200         format(4x,i3,6x,f6.2,6x,f6.2)
            if(ib >= lomo) ibcheck = ibcheck+ib
          endif
        enddo ! jb
        if(ibcheck .ne. ibcheckok)
     &    print *, 'warning: some of the gw corrections are missing'
        
      enddo ! j
      
      print *

      close(13)
      
      return
      end
      
      
!======================================================================
      subroutine wrgw(nk,xk,maxbnd,minbnd,gwen,gwsh,nkibzm)
!======================================================================
!     write GW-energy-parameters
!
!     input:
!----------------------------------------------------------------------
      implicit none
      integer nkibzm
      integer nk
      real xk(3,nk)
      integer minbnd(nk), maxbnd(nk)
      real gwen(nkibzm,*)
      real gwsh(nkibzm,*)
      
      real en
      real gw
      
      integer ik, ib, nb, i
      real, parameter :: HARTREE = 27.2116

      write(*,*) 'writing GW energies and GW corrections'

!     opening file
      open(21,file='out.gw')

!     writing number of k-points
      write(21,'(i6)') nk
      write(*,*) 'number of k-points', nk

      do ik = 1, nk

        write(21,'(3f14.9)') (xk(i,ik),i=1,3)
        nb = maxbnd(ik) - minbnd(ik) + 1
        write(21,'(i2)') nb

        do ib = minbnd(ik), maxbnd(ik)
          en = gwen(ik,ib)*HARTREE
          gw = gwsh(ik,ib)*HARTREE
          write(21,'(i2,3x,f8.4,3x,f8.4)') ib, en, gw
        enddo ! ib
      enddo ! ik
      close(21)
      print *
      
      return
      end

