! Copyright (C) 1998 Valerio Olevano.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

c=======================================================================
c     subroutines for the evaluation of the exchange correlation kernel
c     Kxc in real and trasposed space
c     M. Palummo and V. Olevano 1998
c=======================================================================


c-----------------------------------------------------------------------
c     methods from Kxcjellium(q,n(r)+n(r')/2)
c-----------------------------------------------------------------------

c=======================================================================
      function ckxchubbard(q,n)
c=======================================================================
c     calculate the jellium exchange correlation kernel
c     of given density n and at a given q
c     Kxc(q) = -Vc(q)*G(q)
c     where G(q) is the local field factor in Hubbard expression
c-----------------------------------------------------------------------
      real q
      real n
      parameter(pi = 3.14159265358979324)
      parameter(third=0.33333333333333333)

      qfermi = (3.d0*pi**2*n)**third

      qbig=q/qfermi
      q2big = qbig**2

      if (q.gt.0.001) then 
        ckxchubbard = - (4.*pi/q**2) * 0.5 * q2big/(1+q2big) 
      else 
        ckxchubbard = -2.*pi/qfermi**2
      endif

      return
      end


c=======================================================================
      function ckxcqn(q,n)
c=======================================================================
c     calculate the jellium exchange correlation kernel
c     of given density n and at a given q
c     Kxc(q) = -Vc(q)*G(q)
c     where G(q) is the local field factor in the corradini expression
c-----------------------------------------------------------------------
      real q
      real n
      parameter(third = 1.d0/3.d0)
      parameter(alpha = -0.2846d0)
      parameter(beta1 =  1.0529d0)
      parameter(beta2 =  0.3334d0)
c     parameter(pi = 3.14159265358979324)
      real muc

      pi = 4.d0*datan(1.d0)
      fact = (9.d0*pi/4.d0)**third

      rs =(3.0/(4.0*pi*n))**third
      xx=sqrt(rs)
      qfermi = (3.d0*pi**2*n)**third
      den1 = ( 1.d0 + beta1*xx+beta2*rs)
      ec = alpha / den1
      muc = ec*(1.d0+7.d0*beta1*xx/6.d0+4.d0*beta2*rs/3.d0)/den1
      AA = 0.25d0 +  fact**2*rs**2/18.d0 * (
     &           ec**2*(4.d0*beta2/3.d0 +
     &           7.d0*beta1/(12.d0*xx))/alpha
     &           -2.d0*muc*ec*(beta2 +0.5d0*beta1/xx)/alpha
     &           )
      den2= (3.d0+1.57d0*xx+0.409d0*xx**3)
      BB = (1.d0 + 2.15d0*xx+0.435d0*xx**3)/den2
      CC = 0.25d0*pi*rs*(ec**2*rs*(beta2+0.5d0*beta1/xx)/alpha-ec)/fact
      g = BB/(AA-CC)
      a = (1.5d0/rs**0.25d0) * AA/(BB*G)
      b = 1.2d0/(g*BB)

      qbig=q/qfermi

      if (q.gt.0.000001) then 
        ckxcqn = CC*qbig**2  
     &             + BB*qbig**2/(g+qbig**2)
     &             + a*qbig**4*exp(-b*qbig**2)
        ckxcqn = -4.*pi*ckxcqn/(q)**2
      else 
        ckxcqn = -4.*pi*AA/qfermi**2
      endif

      return
      end


c=======================================================================
      subroutine ckxcnlda(kxct,NPWVEC,gvec,
     &                    nfft1,nfft1a,nfft2,nfft3,
     &                    nr,rho2,qplusG,
     &                    kxcgg)
c=======================================================================
c     calculate the non local density exchange-correlation kernel 
c     in the fourier space Kxcnlda(q,G,G') and put it into kxcgg
c     differenza: lo spazio G e' mantenuto doppio
c     input:
c      rho2(r)
c      qplusG(G) table |q+G|
c     output:
c      kxcgg(G,G') Kxcnlda at q given implicitly in qplusG
c-----------------------------------------------------------------------

      character (len=7) kxct
      logical hubbard, l_stop, l_start
      integer nr
      integer NPWVEC
      real rho2(nr)
      real qplusG(NPWVEC)
      integer gvec(3,NPWVEC)
      complex kxcgg(NPWVEC,NPWVEC)

      complex, allocatable :: kxcrr(:,:)

      hubbard = .false.
      if(kxct.eq.'HUBBARD'.or.kxct.eq.'hubbard') hubbard = .true.
      write(*,*) 'calculating KxcNLDA ', kxct
      write(*,*)

      allocate(kxcrr(nr,nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      do ig=1,npwvec
        do igp=1,npwvec
          kxcgg(ig,igp)=(0.0,0.0)
        enddo
      enddo
      
      igsstart=1
      inquire(file='in.kxc',exist=l_start)
      if(l_start) then
        open(unit=90,file='in.kxc',form='unformatted',status='old')
        read(90) npwvecr, igsstop
        if(npwvecr.ne.npwvec) then
          write(*,*) '** ERROR ckxcnlda 1: wrong dimension in.kxc'
          stop 'in.kxc wrong'
        endif
        write(*,*) 'starting from yet calculated KxcNLDA'
        write(*,*) 'sum term ', igsstop
        igsstart=igsstop+1
        read(90) ((kxcgg(i1,i2),i1=1,npwvec),i2=1,npwvec)
        close(90)
      endif

      inquire(file='kxcstop',exist=l_stop)
      if(l_stop) then
        open(unit=90,file='kxcstop',status='old')
        read(90,*) nstop
        close(unit=90)
        if(igsstart.gt.nstop) goto 4444
      endif


      qmodold = -1.0
      do igs=igsstart,npwvec      

        qmod = qplusG(igs)
c       modificato per test si mette q+G" = 0 
c       qmod = qplusG(1)

c       skip if qmod = qmodold (yet calculated)
        if(abs(qmod-qmodold).lt.0.001) goto 111
        qmodold = qmod

        do ir=1,nr
          do jr=1,nr
            rhom=(rho2(ir)+rho2(jr))*0.5
            if(hubbard) then
              kxcrr(ir,jr) = cmplx(ckxchubbard(qmod,rhom) , 0.0) 
            else
c             'CORRADINI'
              kxcrr(ir,jr) = cmplx(ckxcqn(qmod,rhom) , 0.0) 
            endif
          enddo
        enddo

c       write(6,*) 'kxc(q+G",n(r),n(r1)' 
c       call printm(kxcrr,nr,nr)


c       fft Kxc(q,n(r),n(r')) to Kxc(q,G,G')
        call c6dfft(kxcrr,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)

        rnorm=1./(nfft1*nfft2*nfft3)**2 
        do ig=1,nr
          do igp=1,nr
            kxcrr(ig,igp)=kxcrr(ig,igp)*rnorm  
          end do
        end do 

111     continue

        do ig = 1, NPWVEC
c         form gter = g - gs in fft format
          ig1 = modx(gvec(1,ig)-gvec(1,igs),nfft1)
          ig2 = modx(gvec(2,ig)-gvec(2,igs),nfft2)
          ig3 = modx(gvec(3,ig)-gvec(3,igs),nfft3)
          igt = 1 + ig1 + ig2*nfft1a + ig3*nfft1a*nfft2

          do igp = 1, NPWVEC 
c           form gquater = gp - gs in fft format
            ig1 = modx(gvec(1,igp)-gvec(1,igs),nfft1)
            ig2 = modx(gvec(2,igp)-gvec(2,igs),nfft2)
            ig3 = modx(gvec(3,igp)-gvec(3,igs),nfft3)
            igq = 1 + ig1 + ig2*nfft1a + ig3*nfft1a*nfft2

            kxcgg(ig,igp) = kxcgg(ig,igp) + kxcrr(igt,igq)
          enddo
        enddo

        write(*,*) 'Kxcnlda(q,G,G") sum term ', igs
        write(0,*) 'KxcNLDA(q,G,G") sum term ', igs
        call printcm(kxcgg,NPWVEC,NPWVEC)
        open(unit=90,file='out.kxc',
     &       form='unformatted',status='unknown')
        write(90) npwvec, igs
        write(90) ((kxcgg(i1,i2),i1=1,npwvec),i2=1,npwvec)
        close(90)
        write(*,*) 'kxc written on out.kxc'
        call pclock(1000+igs)
        inquire(file='kxcstop',exist=l_stop)
        if(l_stop) then
          open(unit=90,file='kxcstop',status='old')
          read(90,*) nstop
          close(unit=90)
          if(igs.ge.nstop) goto 4444
        endif

c     close loop on igs
      enddo

4444  continue

      deallocate(kxcrr)

      return
      end 



c=======================================================================
      subroutine ckxcnldad(NPWVEC,gvec,
     &                     nfft1,nfft1a,nfft2,nfft3,
     &                     nr,rho2,qplusG,
     &                     kxcgg,tmp)
c=======================================================================
c     calculate the non local density exchange-correlation kernel 
c     in the fourier space Kxcnlda(q,G,G') and put it into kxcgg
c     Kxc(G,G') = Sum(r,r') Kxc(r,r') exp(-iG.r) exp(+iG'.r')
c     differenza: lo spazio G e' mantenuto doppio
c     input:
c      rho2(r)
c      qplusG(G) table |q+G|
c     output:
c      kxcgg(G,G') Kxcnlda at q given implicitly in qplusG
c     scratch:
c      kxc(r)      temporary nr complex
c-----------------------------------------------------------------------

      integer nr
      integer NPWVEC
      real rho2(nr)
      real qplusG(NPWVEC)
      integer gvec(3,NPWVEC)

      complex kxcgg(NPWVEC,NPWVEC)
      complex tmp(NPWVEC,NPWVEC)


      complex, allocatable :: kxc(:)

      allocate(kxc(nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

c     calculate addresses of workspace arrays for fft
      fftfct2=real((nfft1*nfft2*nfft3)**2) 

      do ig=1,npwvec
        do igp=1,npwvec
          kxcgg(ig,igp)=(0.0,0.0) 
        end do
      end do 

      IRECL = 8
      open(unit=32,status='unknown',access='direct',recl=2*IRECL)
      open(unit=33,status='unknown',access='direct',recl=2*nr*IRECL)
      
      qmodold = -1.0

      do igs = 1, npwvec      

        qmod = qplusG(igs)
c       modificato per test si mette q+G" = 0 
c       qmod = qplusG(1)
        write(*,110) igs, qmod
110     format(i4,f11.6)
        if(abs(qmod-qmodold).lt.0.001) goto 111
        qmodold = qmod

        do jr1 = 0, nfft1-1
        do jr2 = 0, nfft2-1
        do jr3 = 0, nfft3-1
          jr = 1 + jr1 + jr2*nfft1a + jr3*nfft1a*nfft2
          do ir = nfft1a, nr, nfft1a
            kxc(ir)=(0.0,0.0) 
          enddo
          do ir1 = 0, nfft1-1
          do ir2 = 0, nfft2-1
          do ir3 = 0, nfft3-1
            ir = 1 + ir1 + ir2*nfft1a + ir3*nfft1a*nfft2

            rhom=(rho2(ir)+rho2(jr))*0.5
            kxc(ir) = cmplx(ckxcqn(qmod,rhom) , 0.0)

          enddo
          enddo
          enddo

c         fft r -> G with minus sign (exp(-irG))
          call c3dfft(kxc,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
          do ig = 1, nr
            write(32,rec=ig+(jr-1)*nr) kxc(ig)
          enddo

        enddo
        enddo
        enddo


        do ig1 = 0, nfft1-1
        do ig2 = 0, nfft2-1
        do ig3 = 0, nfft3-1
          ig = 1 + ig1 + ig2*nfft1a + ig3*nfft1a*nfft2
          do jr = nfft1a, nr, nfft1a
            kxc(jr)=(0.0,0.0) 
          enddo
          do jr1 = 0, nfft1-1
          do jr2 = 0, nfft2-1
          do jr3 = 0, nfft3-1
            jr = 1 + jr1 + jr2*nfft1a + jr3*nfft1a*nfft2
            read(32,rec=ig+(jr-1)*nr) kxc(jr)
          enddo
          enddo
          enddo
c         for each G fft r' -> G' with plus sign (exp(+irG))
          call c3dfft(kxc,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,+1)
          do jg = 1, nr
            kxc(jg) = kxc(jg)/fftfct2
          enddo
          write(33,rec=ig) (kxc(jg), jg=1,nr)
        enddo
        enddo
        enddo

111     continue

        do ig = 1, NPWVEC
c         form gter = g - gs in fft format
          ig1 = modx(gvec(1,ig)-gvec(1,igs),nfft1)
          ig2 = modx(gvec(2,ig)-gvec(2,igs),nfft2)
          ig3 = modx(gvec(3,ig)-gvec(3,igs),nfft3)
          igt = 1 + ig1 + ig2*nfft1a + ig3*nfft1a*nfft2
          read(33,rec=igt) (kxc(jg), jg=1,nr)
          do igp = 1, NPWVEC 
c           form gquater = gp - gs in fft format
            ig1 = modx(gvec(1,igp)-gvec(1,igs),nfft1)
            ig2 = modx(gvec(2,igp)-gvec(2,igs),nfft2)
            ig3 = modx(gvec(3,igp)-gvec(3,igs),nfft3)
            igq = 1 + ig1 + ig2*nfft1a + ig3*nfft1a*nfft2
            tmp(ig,igp) = kxc(igq)
            kxcgg(ig,igp) = kxcgg(ig,igp) + kxc(igq)
          enddo
        enddo

        write(*,*) 'Kxcnldajellium(q+G",G-G",Gp-G") term G"=',igs
        call printcm(tmp,NPWVEC,NPWVEC)
        write(*,*) 'Kxcnlda(q,G,Gp) sum up to G"=', igs
        call printcm(kxcgg,NPWVEC,NPWVEC)
        open(unit=90,file='out.kxc',
     &       form='unformatted',status='unknown')
        write(90) npwvec
        write(90) ((kxcgg(i1,i2),i1=1,npwvec),i2=1,npwvec)
        close(90)
        write(*,*) 'kxc written on out.kxc'

c     close loop on igs
      enddo
      close(32)
      close(33)

      open(unit=34,file='bu.kxc',form='unformatted')
      write(34) NPWVEC,qplusG(1)
      write(34) ((kxcgg(ig,igp), ig=1,NPWVEC), igp=1,NPWVEC)
      close(34)

      deallocate(kxc)
      return
      end 





c-----------------------------------------------------------------------
c     methods using Kxcjellium(|r-r'|,n(r)+n(r')/2)
c-----------------------------------------------------------------------

c=======================================================================
      subroutine ckxcnldarr(nr,nfft1,nfft1a,nfft2,nfft3,
     &                      a1,a2,a3,
     &                      rho2,kxcrr,kxcr1,BBav,B4pi_gq2f)
c=======================================================================
c     calculate the non local density exchange-correlation kernel 
c     in the real space Kxcnlda(r,r')
c     input:
c      rho2(r)
c     output:
c      kxcrr(r,r') Kxcnlda second and third terms of Corradini expression
c      kxcr1(G)   Kxcnlda(G) first term
c-----------------------------------------------------------------------

      integer nr, nfft1, nfft1a, nfft2, nfft3
      real rho2(nr)
      real a1(3), a2(3), a3(3)

      complex kxcrr(nr,nr)
      complex kxcr1(nr)
      real BBav
      real B4pi_gq2f


      real kxcp,muc

      parameter(third = 1.d0/3.d0)
      parameter(alpha = -0.2846d0)
      parameter(beta1 =  1.0529d0)
      parameter(beta2 =  0.3334d0)
      pi = 4.d0*datan(1.d0)
      fact = (9.d0*pi/4.d0)**third


      write(*,*) 'calculating KxcNLDA(r,r") in real space'
      BBav = 0.0
      B4pi_gq2f = 0.0

      do ir1 = 0, nfft1-1
      do ir2 = 0, nfft2-1
      do ir3 = 0, nfft3-1
        ir = 1 + ir1 + ir2*nfft1a + ir3*nfft1a*nfft2
        do jr1 = 0, nfft1-1
        do jr2 = 0, nfft2-1
        do jr3 = 0, nfft3-1
          jr = 1 + jr1 + jr2*nfft1a + jr3*nfft1a*nfft2

          rhom=(rho2(ir)+rho2(jr))*0.5
          rsm =(3.0/(4.0*PI*rhom))**THIRD
          xx=sqrt(rsm)
          qfermi = (3.d0*pi**2*rhom)**third
          den1 = ( 1.d0 + beta1*xx+beta2*rsm)
          ec = alpha / den1
          muc = ec*(1.d0+7.d0*beta1*xx/6.d0
     &        +4.d0*beta2*rsm/3.d0)/den1
          AA = 0.25d0 +  fact**2*rsm**2/18.d0 * (
     &        ec**2*(4.d0*beta2/3.d0 +
     &         7.d0*beta1/(12.d0*xx))/alpha
     &         -2.d0*muc*ec*(beta2 +0.5d0*beta1/xx)/alpha
     &         )
          den2= (3.d0+1.57d0*xx+0.409d0*xx**3)
          BB = (1.d0 + 2.15d0*xx+0.435d0*xx**3)/den2
          CC = 0.25d0*pi*rsm*(
     &        ec**2*rsm*(beta2+0.5d0*beta1/xx)/alpha
     &      - ec)/fact
          g = BB/(AA-CC)
          a = (1.5d0/rsm**0.25d0) * AA/(BB*G)
          b = 1.2d0/(g*BB)

          q2fermi = qfermi*qfermi
c         calcolare r - r'
          rmrpx = (ir1-jr1)*a1(1)/nfft1 +
     &            (ir2-jr2)*a2(1)/nfft2 +
     &            (ir3-jr3)*a3(1)/nfft3
          rmrpy = (ir1-jr1)*a1(2)/nfft1 +
     &            (ir2-jr2)*a2(2)/nfft2 +
     &            (ir3-jr3)*a3(2)/nfft3
          rmrpz = (ir1-jr1)*a1(3)/nfft1 +
     &            (ir2-jr2)*a2(3)/nfft2 +
     &            (ir3-jr3)*a3(3)/nfft3
          rmrp2 = rmrpx*rmrpx+rmrpy*rmrpy+rmrpz*rmrpz
          rmrp =  sqrt(rmrp2)

c         second term of corradini's expression
          kxcp = (a*qfermi/(4.0*sqrt(pi)*b**2.5)) *
     &           (q2fermi*rmrp2*0.5/b - 3.0) *
     &           exp(-q2fermi*rmrp2*0.25/b)

          if(ir.ne.jr) then
c           third term first part of corradini's expression
            kxcp = kxcp - BB*(exp(-sqrt(g)*qfermi*rmrp)-1.0)/rmrp
          else
            kxcp = kxcp + BB*sqrt(g)*qfermi

c           first term (diagonal) of corradini's expression
            kxcr1(ir) = -4*pi*CC/q2fermi
          endif

          kxcrr(ir,jr) = cmplx(kxcp,0.0) 

c         B average to treat third term second part
          BBav = BBav + BB
c         B4pi_gq2f to treat the case q=0, G=G'=0
          B4pi_gq2f = B4pi_gq2f + BB/(g*q2fermi)

        enddo
        enddo
        enddo
      enddo
      enddo
      enddo

c     write(*,*) 'Kxc(r,r")' 
c     call printmr(kxcrr,nfft1,nfft1a,nfft2,nfft3,nr)

c     now fft kxcr1     
      write(*,*) 'fft KxcNLDA(r-r") -> KxcNLDA(G) 1st term'
      call c3dfft(kxcr1,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
      fftfct = real(nfft1*nfft2*nfft3)
      do ir = 1, nr
        kxcr1(ir) = kxcr1(ir)/fftfct
      enddo
c     now we have kxcr1 (first term of corradini expression)
c       in kxcr1(G''' = G"-G') fft format

c     B average to treat the third term second part
      BBav = BBav/(fftfct*fftfct)
      B4pi_gq2f = B4pi_gq2f/(fftfct*fftfct)

      write(*,*)
      return
      end 



      subroutine ckxcnldagg(Kxcnldarr,Kxcnldagg,q,
     &                      nr,nfft1,nfft1a,nfft2,nfft3,
     &                      NPWVEC,a1,a2,a3,b1,b2,b3,tpula,
     &                      igfft)

      integer nr, nfft1, nfft1a, nfft2, nfft3, NPWVEC
      complex Kxcnldarr(nr,nr)
      complex Kxcnldagg(NPWVEC,NPWVEC)
      real tpula
      real q(3)
      real a1(3), a2(3), a3(3)
      real b1(3), b2(3), b3(3)
      integer igfft(NPWVEC,5,5,5)
      real qx, qy, qz

      complex, allocatable :: ctemp(:,:)

      allocate(ctemp(nr,nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      qx = (q(1)*b1(1) + q(2)*b2(1) + q(3)*b3(1))
      qy = (q(1)*b1(2) + q(2)*b2(2) + q(3)*b3(2))
      qz = (q(1)*b1(3) + q(2)*b2(3) + q(3)*b3(3))

      do ir1 = 0, nfft1-1
      do ir2 = 0, nfft2-1
      do ir3 = 0, nfft3-1
        ir = 1 + ir1 + ir2*nfft1a + ir3*nfft1a*nfft2
        do jr1 = 0, nfft1-1
        do jr2 = 0, nfft2-1
        do jr3 = 0, nfft3-1
          jr = 1 + jr1 + jr2*nfft1a + jr3*nfft1a*nfft2

c         calcolare |r - r'|
          rmrpx = (ir1-jr1)*a1(1)/nfft1 +
     &            (ir2-jr2)*a2(1)/nfft2 +
     &            (ir3-jr3)*a3(1)/nfft3
          rmrpy = (ir1-jr1)*a1(2)/nfft1 +
     &            (ir2-jr2)*a2(2)/nfft2 +
     &            (ir3-jr3)*a3(2)/nfft3
          rmrpz = (ir1-jr1)*a1(3)/nfft1 +
     &            (ir2-jr2)*a2(3)/nfft2 +
     &            (ir3-jr3)*a3(3)/nfft3

          qrmrp = qx*rmrpx + qy*rmrpy + qz*rmrpz

          ctemp(ir,jr) = Kxcnldarr(ir,jr)*exp(-(0.0,1.0)*qrmrp)
        enddo
        enddo
        enddo
      enddo
      enddo
      enddo

      call c6dfft(ctemp,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)

      fftfct = nfft1*nfft1*nfft2*nfft2*nfft3*nfft3
      do ig = 1, NPWVEC
        do jg = 1, NPWVEC
c         Kxcnldagg(ig,jg) = Kxcnldagg(ig,jg)/fftfct
          Kxcnldagg(ig,jg) = ctemp(igfft(ig,3,3,3),igfft(jg,3,3,3))
     &                       /fftfct
        enddo
      enddo

!     remove tpula from subroutine
      ctemp(1,1) = tpula
      deallocate(ctemp)
      return
      end




!     ==================================================================
      subroutine c6dfft(a,id,ie,if,nl,nm,nn,isig)
!     ==================================================================
!     calculate a 6-dimensional Fast Fourier Transform
!
!     isig = -1 : A(G,G') = Sum(r,r') A(r,r') exp(-iG.r) exp(+iG'.r')
!                                                 ^          ^
!     isig = +1 : A(r,r') = Sum(G,G') A(G,G') exp(+iG.r) exp(-iG'.r')
!                                                 ^          ^
!     A(G,G') is returned as NR complex numbers; other coefficients
!     are discarded.  A(r,r') is supplied as NR**2
!     complex numbers where NR is the dimension of the 3D FFT.
!
!     isig=-1 and isig=1 form a transform/inverse-transform pair: calling
!     one then the other will take you back to the original function,
!     multiplied by a factor of (nl*nm*nn)**2.
!     ------------------------------------------------------------------
!
!     input:
!      a: A(r,r') [overwritten]
!     output:
!      a: A(G,G') in the format IGFFT
!     ------------------------------------------------------------------

      complex a(id*ie*if,id*ie*if)
      complex, allocatable :: w(:)

      nr = id*ie*if

      allocate(w(nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

!     first do the r->G transform for each r' to form A(G,r')
      do ixp = 1, nl
      do iyp = 1, nm
      do izp = 1, nn
        irp = ixp + id*(iyp-1) + id*ie*(izp-1)
        call c3dfft(a(1,irp),id,ie,if,nl,nm,nn,isig)
      enddo
      enddo
      enddo

!     now do the r'->G' transform for each G
      do ix = 1, nl
      do iy = 1, nm
      do iz = 1, nn
        ig = ix + id*(iy-1) + id*ie*(iz-1)

        do irp = 1, nr
          w(irp) = a(ig,irp)
        enddo

        call c3dfft(w,id,ie,if,nl,nm,nn,-isig)

        do igp = 1, nr
          a(ig,igp) = w(igp)
        enddo

      enddo
      enddo
      enddo

      deallocate(w)
      return
      end

