! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

subroutine setmesh(ng,nw,nm,gvec,alias,addshell,nfft,igfft)
  implicit none
  integer ng, nw, nm, nfft(3)
  integer alias, addshell
  integer gvec(3,ng), igfft(ng,-2:2,-2:2,-2:2)
  integer mw(3), mm(3), nfftb(3)
  integer ig, ig01, ig02, ig03, gmg01, gmg02, gmg03
  integer, external :: allowed_fftsize, modx

  print *, 'selecting fft mesh size'
  print *, 'alias = ', alias
! find the mesh dimension corresponding to nw=npwwfn
  mw(1) = maxval(abs(gvec(1,1:nw)))
  mw(2) = maxval(abs(gvec(2,1:nw)))
  mw(3) = maxval(abs(gvec(3,1:nw)))
! in case we want to avoid the fft aliasing problem, double it
  if(alias==2) mw(:) = mw(:) * 2

! find the mesh dimension corresponding to nm=npwmat (arrival values)
  mm(1) = maxval(abs(gvec(1,1:nm)))
  mm(2) = maxval(abs(gvec(2,1:nm)))
  mm(3) = maxval(abs(gvec(3,1:nm)))
! in case the arrival value is required with +- a g0 to bring it into the 1BZ
  if(addshell==1 .or. addshell==2) mm(:) = mm(:) + addshell

! find the fft mesh which is -m,...,+m that is 2m+1
  nfftb(1) = 2 * max(mw(1),mm(1)) + 1
  nfftb(2) = 2 * max(mw(2),mm(2)) + 1
  nfftb(3) = 2 * max(mw(3),mm(3)) + 1
! verify if are allowed by the fft library, otherwise chose larger values
  nfft(1) = allowed_fftsize(nfftb(1))
  nfft(2) = allowed_fftsize(nfftb(2))
  nfft(3) = allowed_fftsize(nfftb(3))
  write(*,'(" mesh size selected: ",i3," x",i3," x",i3)') nfft(:)
  write(*,'(" total size, nr = ",i6)') nfft(1)*nfft(2)*nfft(3)

  if(any(mw(:)<mm(:))) then
    print *, 'WARNING: the fft mesh size selected must be taken larger than'
    print *, '         the value corresponding to npwwfn.'
    print *, '         You could increase npwwfn without lost in cpu time.'
    print *
  endif

  do ig01 = -2, 2
    do ig02 = -2, 2
      do ig03 = -2, 2
        do ig = 1, ng
          gmg01 = modx( gvec(1,ig) - ig01 , nfft(1) )
          gmg02 = modx( gvec(2,ig) - ig02 , nfft(2) )
          gmg03 = modx( gvec(3,ig) - ig03 , nfft(3) )
          igfft(ig,ig01,ig02,ig03)=1+gmg01+gmg02*nfft(1)+gmg03*nfft(1)*nfft(2)
       enddo
     enddo
   enddo
  enddo
  print *, 'fft tables setup'
  print *
  return
end

