/*
Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
All rights reserved.

DP code
Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
Calculate Dielectric Properties.
*/

/* 
file rdcl.c

contains common routines to read command line for arguments
to be passed to programs dp, exc, tau

*/


#include <stdio.h>
#include <string.h>



read_command_line(argc,argv,inputfile,kssfile,gwfile,em1file,vkbfile)
int argc;
char *argv[];
char **inputfile, **kssfile, **gwfile, **em1file, **vkbfile;
{
  int i;
  char *program, *control;
  
  program = argv[0];
  while(--argc>0) {
    argv++;
    if(*argv[0]!='-') {
      /* default argument: kss file */
      *kssfile = *argv;
    } else {
      i=0;
      control = *argv;
      while(control[++i]!='\0') {
        switch(control[i]) {
	  case 'i':
	    if(--argc<0) error_missing_argument(program);
	    *inputfile = *++argv;
	    break;
	  case 'q':
          case 'k':
	    if(--argc<0) error_missing_argument(program);
	    *kssfile = *++argv;
	    break;
	  case 'g':
	    if(--argc<0) error_missing_argument(program);
	    *gwfile = *++argv;
 	    break;
	  case 'v':
	    if(--argc<0) error_missing_argument(program);
	    *vkbfile = *++argv;
 	    break;
	  case 'e':
	    if(--argc<0) error_missing_argument(program);
	    *em1file = *++argv;
 	    break;
	  default:
            error_option(program,control[i]);
        }
      }
    }
  }
  return(1);    
}

check_file(filename,defaultname,program)
char *program, **filename, *defaultname;
/* return file length if filename or defaultname exists, otherwise 0 */ 
{
  int nl;
  FILE *fp;
  if(*filename!=NULL) {
    nl = strlen(*filename);
    fp = fopen(*filename,"r");
    if(fp==NULL) error_file(program,*filename);
    fclose(fp);
  } else {
    *filename = defaultname; /* default */
    nl = strlen(*filename);
    fp = fopen(*filename,"r");
    if(fp!=NULL) close(fp);
    else *filename=NULL, nl=0;
  }
  return(nl);
}

error_option(program,c)
char *program;
char c;
/* print on stderr option error */
{
  fprintf(stderr,"%s: illegal option --%c\n",program,c);
  usage(program); 
  exit(2);
}

error_missing_argument(program)
char *program;
/* print on stderr missing argument error */
{
  fprintf(stderr,"%s: missing argument\n",program);
  usage(program); 
  exit(3);
}

error_file(program,filename)
char *program, *filename;
/* print on stderr missing file */
{
  fprintf(stderr,"%s: not existing or not readable file %s\n",
    program,filename);
  usage(program);
  exit(4);
}

usage(program)
char *program;
/* print on stderr usage options */
{
  fprintf(stderr,
    "usage: %s [-i inputfile] [-g gwfile] [-e em1file] [-k] kssfile\n",
    program); 
  fprintf(stderr,"obsolescences: [-q qpldafile] [-v vkbfile]\n");
  fprintf(stderr,"default: %s.in in.kss in.gw in.em1\n",program);
}  
