! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

      subroutine cden(nbnds,nkibz,nkbz,nop,ninv,nr,
     &                   nfft1,nfft1a,nfft2,nfft3,
     &                   irottb,ucvol,wtk,occ,wfr,rho2,omegaplasma)
      implicit none
      integer nbnds, nop, ninv
      integer nkibz, nkbz, nr, nfft1, nfft1a, nfft2, nfft3
      real ucvol
      real occ(nkibz,nbnds)
      real wtk(nkibz)
      integer irottb(nr,nop)
      complex wfr(nr,nbnds,nkibz)
      real rho2(nr)
      real rho(nr)
      integer ikibz, ir, ib, iop
      real fact

      real tnepuc, rhoav, rs, omegaplasma
      real, parameter :: HARTREE = 27.2116

      print *, 'calculating charge density'

      rho(:) = 0.0
      rho2(:) = 0.0

      do ikibz = 1, nkibz
        do ib = 1, nbnds
          if (occ(ikibz,ib)< 1.e-5) exit
          rho(:) = rho(:) + occ(ikibz,ib) * wtk(ikibz) *
     &             conjg(wfr(:,ib,ikibz)) * wfr(:,ib,ikibz)
        enddo
      enddo

!     loop over symmetry operations, symmetrising rho
      if(ninv == 1) then
        fact = 1.0 / (nkbz * ucvol)
      else
!       factor of 2 is for inversion
        fact = 2.0 / (nkbz * ucvol)
      endif
      do iop = 1, nop
        do ir = 1, nr
          rho2(ir)=rho2(ir)+fact*rho(irottb(ir,iop))
        enddo
      enddo

c     write total charge:
c     open(unit=66,file='rho')
c     write(66,1000) (rho2(ir), ir=1,nr)
c1000  format(f7.4)
c     close(66)


c     calculate total number of electrons as a check
      tnepuc = 0.0
      do ir = 1, nr
        tnepuc = tnepuc + rho2(ir)
      enddo
      tnepuc = tnepuc * ucvol / (nfft1*nfft2*nfft3)
      write (6,1110) tnepuc
1110  format (' total number of electrons per unit cell = ',f7.4)
      rhoav = tnepuc / ucvol
      write(6,1115) rhoav
1115  format(' average of density, n = ',f7.4)
      rs = (3.0/(4.0*(4.0*atan(1.0))*rhoav))**(1.0/3.0)
      write(6,1120) rs
1120  format(' r_s = ',f7.4)
      omegaplasma = sqrt(4.0 * (4.0*atan(1.0)) * rhoav)
      write(6,1125) omegaplasma*HARTREE
1125  format(' omega_plasma = ',f7.4, '[eV]')
      write(*,*)

      return
      end

