! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
       subroutine rho_tw_g(NPWVEC,nr,nrb,nfft1a,nfft1,nfft2,nfft3,
     &    igfftg0,wfn1,i1,ktabr1,wfn2,i2,ktabr2,rhotwg)
!     ==================================================================
!     calculates rhotwg(G) = < wfn1 | exp(-i*(q+G)*r) | wfn2 >
!     ------------------------------------------------------------------
      implicit none
      integer nr, npwvec
      integer nrb
      integer i1, i2
      integer nfft1,nfft1a,nfft2,nfft3
      integer igfftg0(NPWVEC)
      integer ktabr1(nr) ! tables R^-1 r for that k points
      integer ktabr2(nr)
      complex wfn1(nr)
      complex wfn2(nr)
      complex rhotwg(NPWVEC)
      complex rhotw(nr)
      integer ir
!     to account for symmetries
!     phi_Rk(r) = phi_k(R^-1 r)
!     phi_Ik(r) = phi_k*(r)
!     form rho-twiddle(r)
      if(i1 == 1) then
        if(i2 == 1) then
          do ir = 1, nr
            rhotw(ir) = conjg(wfn1(ktabr1(ir))) * wfn2(ktabr2(ir))
          enddo
        else
          do ir = 1, nr
            rhotw(ir) = conjg(wfn1(ktabr1(ir)) * wfn2(ktabr2(ir)))
          enddo
        endif
      else
        if(i2 == 1) then
          do ir = 1, nr
            rhotw(ir) = wfn1(ktabr1(ir)) * wfn2(ktabr2(ir))
          enddo
        else
          do ir = 1, nr
            rhotw(ir) = wfn1(ktabr1(ir)) * conjg(wfn2(ktabr2(ir)))
          enddo
        endif
      endif
!     FFT to rho-twiddle (G)
      call c3dfft(rhotw,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
!     Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
      rhotwg(:) = rhotw(igfftg0(:)) / nrb
      return
      end



!     ==================================================================
      subroutine crhotwiddleg(NPWVEC,nr,nrb,nfft1a,nfft1,nfft2,nfft3,
     &                        igfftg0,wfn1,wfn2,rhotwg)
!     ==================================================================
!     calculates rhotwg(G) = < wfn1 | exp(-i*(q+G)*r) | wfn2 >
!     without symmetry operations and inversion
!     ------------------------------------------------------------------
      implicit none
      integer nr, NPWVEC
      integer nrb
      integer nfft1,nfft1a,nfft2,nfft3
      integer igfftg0(NPWVEC)
      complex wfn1(nr), wfn2(nr)
      complex rhotwg(NPWVEC)
      complex rhotw(nr)
!     form rho-twiddle(r)
      rhotw(:) = conjg(wfn1(:)) * wfn2(:)
!     FFT to rho-twiddle(G)
      call c3dfft(rhotw,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
!     Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
      rhotwg(:) = rhotw(igfftg0(:)) / nrb
      return
      end




