! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! Contributor: Fabien Bruneval
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
      subroutine ccgradvnl(npwwfn,nkibz,b1,b2,b3,gvec,kibz,ntype,natom,
     &                     mpsang,type,xcart,vkbsign,vkb,vkbd,gradvnl)
!     ==================================================================
!     compute the (grad_K + grad_K') Vnl(K,K')
!     three reciprocal lattice units components
!     input:
!       vkb and vkbd kleinman-bylander pseudopotential in separable form
!         read from unit 11
!     ------------------------------------------------------------------
      implicit none
      logical, parameter :: DEBUG = .false.
      integer npwwfn, nkibz
      real b1(3), b2(3), b3(3)
      integer gvec(3,npwwfn)
      real kibz(3,nkibz)
      integer ntype, natom, mpsang
      integer type(natom)
      double precision :: xcart(3,natom)
      real vkbsign(mpsang,ntype)
      real vkb(npwwfn,ntype,mpsang,nkibz)
      real vkbd(npwwfn,ntype,mpsang,nkibz)
      
!     (grad_K + grad_K') Vnl(K,K') in reciprocal lattice units
      complex gradvnl(3,npwwfn,npwwfn,nkibz)

!     kleinman-bylander non-local separable forms and structure factors
!     number of atomic species ntype
!     number of angular momentum components in pseudopotential mpsang
      complex sfac(ntype)
      real taugd

!     kg = k + G, kgp = k + G', moduli and various products
      real kg(3), kgp(3)
      real (kind(0.d0)) :: mkg, mkgp, mkg2, mkgp2, mkgkgp, costheta
      real (kind(0.d0)) :: x, x2, x3, x4, x5, x6, x7

      integer lmax

      integer i, ig, igp, ik, ia, is, il
      integer igd1, igd2, igd3
      real rgdx, rgdy, rgdz
!     temporary complex
      complex ct, cs

      real, external :: scpdt

!     legendre polinomials and their first derivatives 
!     s p d f g h i j  so up to PL_7 = pl(8) 
      integer, parameter :: nlx = 8
      real (kind(0.d0)) :: pl(nlx), pld(nlx)

!     legendre polinomials and their derivatives
      pl(1) = 1.d0
      pld(1) = 0.d0
!     pl(2) = costheta
      pld(2) = 1.d0
!     pl(3) = 1/2 ( 3 costheta**2 - 1 )
!     pld(3) = 3 costheta

      print *, 'limit q->0, including term <n,k|[Vnl,iqr]|n"k>'

      lmax = mpsang
      if(mpsang>nlx) then
        print *, 'warning: number of angular momentum components'
        print *, '         bigger than programmed.'
        print *, '         taken into account only s p d f g h i j'
        lmax = nlx
      endif

      do ia = 1, natom
        if(type(ia) < 1 .or. type(ia) > ntype) stop 'BUG on type'
      enddo

      if(DEBUG) then
        print *, 'mpsang ',  mpsang
        print *, 'ntype ', ntype, 'natom ', natom
        print *, 'type ', type
        print *, 'vkbsign ', vkbsign
        print *, 'vkb(1:3,1,1,1) ', vkb(1:3,1,1,1)
        print *, 'vkbd(1:3,1,1,1) ', vkbd(1:3,1,1,1)
        print *, 'xcart ', xcart
      endif

      gradvnl(:,:,:,:) = (0.0,0.0)

      do ik = 1, nkibz
      
       do ig = 1, npwwfn
        kg(:) = kibz(:,ik) + real(gvec(:,ig))
        mkg2 = scpdt(kg,kg,b1,b2,b3)
        mkg = sqrt(mkg2)
!       the next to solve the problem with k=Gamma (may be)
        if(mkg < 0.0001) cycle
        do igp = 1, npwwfn
          kgp(:) = kibz(:,ik) + real(gvec(:,igp))
          mkgp2 = scpdt(kgp,kgp,b1,b2,b3)
          mkgp = sqrt(mkgp2)         
!         the next to solve the problem with k=Gamma (may be)
          if(mkgp < 0.0001) cycle

          mkgkgp = mkg*mkgp
          costheta = scpdt(kg,kgp,b1,b2,b3) / mkgkgp
!         here if x is small, you can have
!         forrtl: error (74): floating underflow
!         before the 7th power
          x = costheta
          x2 = x * x
          x3 = x2 * x
          x4 = x3 * x
          x5 = x4 * x
          x6 = x5 * x
          x7 = x6 * x

!         calculate legendre polinomial PL_0 = pl(1)
          pl(2) = x
          pl(3) = (3.0/2.0) * x2 - (1.0/2.0)
          pl(4) = (5.0/2.0) * x3 - (3.0/2.0) * x
          pl(5) = (35.0/8.0) * x4 - (30.0/8.0) * x2 + (3.0/8.0)
          pl(6) = (63.0/8.0) * x5 - (70.0/8.0) * x3 + (15.0/8.0) * x
          pl(7) = (231.0/16.0) * x6 - (315.0/16.0) * x4 + 
     &            (105.0/16.0) * x2 - (5.0/16.0)
          pl(8) = (429.0/16.0) * x7 - (693.0/16.0) * x5 +
     &            (315.0/16.0) * x3 - (35.0/16.0) * x
!         calculate legendre polinomial derivative
          pld(3) = 3.0 * x
          pld(4) = (15.0/2.0) * x2 - (3.0/2.0)
          pld(5) = (35.0/2.0) * x3 - (15.0/2.0) * x
          pld(6) = (315.0/8.0) * x4 - (210.0/8.0) * x2 + (15.0/8.0)
          pld(7) = (693.0/8.0) * x5 - (315.0/4.0) * x3 + (105.0/8.0) * x
          pld(8) = (3003.0/16.0) * x6 - (3465.0/16.0) * x4 +
     &             (945.0/16.0) * x2 - (35.0/16.0)

          igd1 = gvec(1,ig)-gvec(1,igp) 
          igd2 = gvec(2,ig)-gvec(2,igp) 
          igd3 = gvec(3,ig)-gvec(3,igp) 
          rgdx = igd1*b1(1)+igd2*b2(1)+igd3*b3(1)
          rgdy = igd1*b1(2)+igd2*b2(2)+igd3*b3(2)
          rgdz = igd1*b1(3)+igd2*b2(3)+igd3*b3(3)
          do is = 1, ntype
            sfac(is) = (0.0,0.0)
            do ia = 1, natom
              if(type(ia)/=is) cycle
              taugd = rgdx*xcart(1,ia)+rgdy*xcart(2,ia)+
     &                rgdz*xcart(3,ia)
              sfac(is) = sfac(is) + cmplx(cos(taugd),-sin(taugd))
            enddo
          enddo

          do i = 1, 3
            gradvnl(i,ig,igp,ik) = 0.0
            do is = 1, ntype
              do il = 1, lmax
                ct = ( kg(i)*(1/mkgkgp - costheta/mkg2 ) +
     &                 kgp(i)*(1/mkgkgp - costheta/mkgp2 ) ) *
     &               pld(il) * 
     &             vkbsign(il,is) * vkb(ig,is,il,ik) * vkb(igp,is,il,ik)
                cs = pl(il) * vkbsign(il,is) *
     &             ( kg(i)/mkg * vkbd(ig,is,il,ik) * vkb(igp,is,il,ik) +
     &             kgp(i)/mkgp * vkb(ig,is,il,ik) * vkbd(igp,is,il,ik) )
                gradvnl(i,ig,igp,ik) = gradvnl(i,ig,igp,ik) + 
     &                              sfac(is) * (ct + cs)
              enddo ! il
            enddo ! is
          enddo ! i
          
        enddo ! igp
       enddo ! ig
      enddo ! ik

      if(DEBUG) then
        print *, 'gradvnl '
        call printcm(gradvnl(1,:,:,1),npwwfn,npwwfn)
        call printcm(gradvnl(2,:,:,1),npwwfn,npwwfn)
        call printcm(gradvnl(3,:,:,1),npwwfn,npwwfn)
      endif      

      return
      end


!     ==================================================================
      subroutine cfnl(npwwfn,nkibz,a1,a2,a3,b1,b2,b3,gvec,
     &                     kibz,ntype,natom,
     &                 mpsang,type,xcart,vkbsign,vkb,vkbd,kb)
!     ==================================================================
!     compute Vnl(K) and grad_K Vnl(K)
!     three reciprocal lattice units components
!     Fabien Bruneval march 2003
!     input:
!       vkb and vkbd kleinman-bylander pseudopotential in separable form
!     ------------------------------------------------------------------
      use ggwdef
      implicit none
      logical, parameter :: DEBUG = .false.
      integer npwwfn, nkibz
      real a1(3), a2(3), a3(3)
      real b1(3), b2(3), b3(3)
      integer gvec(3,npwwfn)
      real kibz(3,nkibz)
      integer ntype, natom, mpsang
      integer type(natom)
      real (kind(0.d0)) :: xcart(3,natom)

!     kleinman-bylander non-local separable forms and structure factors
      real vkbsign(mpsang,ntype)
      real vkb(npwwfn,ntype,mpsang,nkibz)
      real vkbd(npwwfn,ntype,mpsang,nkibz)
      complex, allocatable :: l_fnl(:,:,:,:), l_fnld(:,:,:,:,:)

!     Kleiman-Bylander factor and first derivative      
!     example: fnl(ig,im+(il-1)*(il-1),is,ik)
!     example: fnld (i,ig,im+(il-1)*(il-1),is,ik)
!      complex  :: fnl(npwwfn,mpsang*mpsang,natom,nkibz),
!     &            fnld(3,npwwfn,mpsang*mpsang,natom,nkibz)
      type(kb_potential) kb

      complex (kind(0.d0)) :: sfac
      complex (kind(0.d0)) :: dylmcart(3), dylmcrys(3), dth, dphi,
     &                          gradth(3), gradphi(3)
      real (kind(0.d0)) :: costh, sinth, cosphi, sinphi, sq
      real (kind(0.d0)) :: xdotg

!     kg = k + G in crystallographic coordinates
      real (kind(0.d0)) :: kg(3)
      real (kind(0.d0)) :: mkg, mkg2
      real (kind(0.d0)) :: pig, factor
      integer lmax

      integer i, j, ig, ik, ia, is, il, im, iml
      real (kind(0.d0)) :: kcart(3), gcart(3)
      integer istat

      complex, external :: ylm

!     spherical harmonics and their first derivatives 
      integer, parameter :: nlx = 3

      pig = dacos(-1.d0)
      print *, 'limit q->0, including term <n,k|[Vnl,iqr]|n"k>'

      allocate(kb%fnl(npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
      if(istat/=0) stop 'out of memory'
      allocate(kb%fnld(3,npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
      if(istat/=0) stop 'out of memory'
      allocate(l_fnl(npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
      if(istat/=0) stop 'out of memory'
      allocate(l_fnld(3,npwwfn,mpsang*mpsang,natom,nkibz),stat=istat)
      if(istat/=0) stop 'out of memory'
      
      lmax = mpsang
      if(mpsang>nlx) then
        print *, 'warning: number of angular momentum components'
        print *, '         bigger than programmed.'
        print *, '         taken into account only s p d' 
        lmax = nlx
      endif

      if(DEBUG) then
        print *, 'mpsang ',  mpsang
        print *, 'ntype ', ntype, 'natom ', natom
        print *, 'type ', type
        print *, 'vkbsign ', vkbsign
        print *, 'vkb(1:3,1,1,1) ', vkb(1:3,1,1,1)
        print *, 'vkbd(1:3,1,1,1) ', vkbd(1:3,1,1,1)
        print *, 'nikbz', nkibz
        print *, 'xcart ', xcart
        print *, 'a1', a1(:)
        print *, 'b1', b1(:)
      endif

      do ik = 1, nkibz
       do ig = 1, npwwfn
        kg(:) = kibz(:,ik) + real(gvec(:,ig))
        kcart(:) = kg(1)*b1(:)+kg(2)*b2(:)+kg(3)*b3(:)
!       the next to solve the problem with sinth=0. or sinphi=0.
        if(dabs(kcart(2))<1.d-8) kcart(2) = kcart(2)+1.d-8
        mkg2 = kcart(1)**2+kcart(2)**2+kcart(3)**2
        mkg = dsqrt(mkg2)
        sq = dsqrt(kcart(1)**2+kcart(2)**2)

        gcart(:) = real(gvec(1,ig))*b1(:)
     &            +real(gvec(2,ig))*b2(:)
     &            +real(gvec(3,ig))*b3(:)

! (th,phi) spherical coordinates
        costh = kcart(3)/mkg
        sinth = sq/mkg
        cosphi = kcart(1)/sq
        sinphi = kcart(2)/sq
        gradth(1) = kcart(1)*kcart(3)/mkg**3/sinth
        gradth(2) = kcart(2)*kcart(3)/mkg**3/sinth
        gradth(3) = -(1.d0/mkg-kcart(3)**2/mkg**3)/sinth
        gradphi(1) = -(1.d0/sq - kcart(1)**2/sq**3)/sinphi
        gradphi(2) = kcart(2)*kcart(1)/sq**3/sinphi
        gradphi(3) = (0.d0,0.d0)

        do ia = 1, natom
         is=type(ia)
         xdotg = gcart(1)*xcart(1,ia)+gcart(2)*xcart(2,ia)+
     &             gcart(3)*xcart(3,ia)
         sfac = cmplx(dcos(xdotg),dsin(xdotg),kind(0.d0)) 

         do il = 1, lmax
          factor = dsqrt(4.d0*pig/real(2*(il-1)+1,kind(0.d0)))
          do im= 1, 2*(il-1)+1
!  index on im and il
            iml = im+(il-1)*(il-1)

!  first k-b factor
            l_fnl(ig,iml,ia,ik) = factor * sfac
     &             * ylm(im-il,il-1,kcart)
     &             * vkb(ig,is,il,ik) * vkbsign(il,is)

!  second k-b factor (involving first derivatives)
            call ylmd(im-il,il-1,kcart,dth,dphi)
!  dYlm/dK = dYlm/dth * grad_K th + dYlm/dphi + grad_K phi
            dylmcart(:) = dth * gradth(:) + dphi * gradphi(:)
!  cartesian to crystallographic axis
            dylmcrys(:) = ( dylmcart(1)*a1(:)
     &                     +dylmcart(2)*a2(:)
     &                     +dylmcart(3)*a3(:) ) /(2.d0*pig)
            do i = 1, 3
             l_fnld(i,ig,iml,ia,ik) = factor * sfac
     &         *( kg(i)/mkg * ylm(im-il,il-1,kcart) * vkbd(ig,is,il,ik)
     &           + dylmcrys(i) * vkb(ig,is,il,ik) )
            enddo ! i

          enddo ! im
         enddo ! il
        enddo ! ia
       enddo ! ig
      enddo ! ik

      kb%fnl(:,:,:,:)=l_fnl(:,:,:,:)
      kb%fnld(:,:,:,:,:)=l_fnld(:,:,:,:,:)
      deallocate(l_fnl,l_fnld)
      return
      end


!     ==================================================================
      complex function ylm(im,il,kcart)
!     ==================================================================
!
!     This function gives all spherical harmonics for il<=2
!     
      implicit none
!  input variables 
      integer im, il
      real (kind(0.d0)) :: kcart(3)
!  local variables
      real (kind(0.d0)) :: pig
      real (kind(0.d0)) :: costh, sinth, 
     &                     cosphi, sinphi, costwophi, sintwophi

      pig = dacos(-1.d0)

!     (th,phi) spherical coordinates
      costh = kcart(3)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
      sinth = dsqrt(kcart(1)**2+kcart(2)**2)
     &                    /dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
      cosphi = kcart(1)/dsqrt(kcart(1)**2+kcart(2)**2)
      sinphi = kcart(2)/dsqrt(kcart(1)**2+kcart(2)**2)
      costwophi = 2.d0*cosphi**2 - 1.d0
      sintwophi = 2.d0*sinphi*cosphi

      if (il.eq.0) then
       if (abs(im).eq.0) then
        ylm = 1.d0 / dsqrt(4.d0*pig)
       else
        stop 'Bug ylm'
       endif
      elseif (il.eq.1) then
       if (abs(im).eq.0) then
        ylm = dsqrt(3.d0/(4.d0*pig))*costh
       elseif (abs(im).eq.1) then
        ylm = -dsqrt(3.d0/(8.d0*pig))*sinth*cmplx(cosphi,sinphi)
       else
        stop 'Bug ylm'
       endif
      elseif (il.eq.2) then
       if (abs(im).eq.0) then
        ylm = dsqrt(5.d0/(16.d0*pig))*(3.d0*costh**2-1.d0)
       elseif (abs(im).eq.1) then
        ylm = -dsqrt(15.d0/(8.d0*pig))
     &            *sinth*costh*cmplx(cosphi,sinphi)
       elseif (abs(im).eq.2) then
        ylm = dsqrt(15.d0/(32.d0*pig))*(sinth)**2
     &            *cmplx(costwophi,sintwophi)
       else
        stop 'Bug ylm'
       endif
      else
       stop 'Ylm not yet implemented'
      endif

!  treats the case im < 0
      if (im.lt.0) then
       ylm=(-1.d0)**(im)*conjg(ylm)
      endif

      end function

!     ==================================================================
      subroutine ylmd(im,il,kcart,dth,dphi)
!     ==================================================================
!
!     This subroutine computes dth and dphi, the first derivatives of
!     Ylm as a function of th and phi (the angles of the spherical coordinates)
!     It works for all spherical harmonics with il <=2
!
      implicit none

!  input variables
      integer im, il
      real (kind(0.d0)) :: kcart(3)
!  output variable
      complex (kind(0.d0)) :: dth, dphi
!  local variables
      real (kind(0.d0)) :: pig
      real (kind(0.d0)) :: costh, sinth,
     &                     cosphi, sinphi, costwophi, sintwophi

      pig = dacos(-1.d0)
!     (th,ph) spherical representation.
      costh = kcart(3)/dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
      sinth = dsqrt(kcart(1)**2+kcart(2)**2)
     &                    /dsqrt(kcart(1)**2+kcart(2)**2+kcart(3)**2)
      cosphi = kcart(1)/dsqrt(kcart(1)**2+kcart(2)**2)
      sinphi = kcart(2)/dsqrt(kcart(1)**2+kcart(2)**2)
      costwophi = 2.d0*cosphi**2 - 1.d0
      sintwophi = 2.d0*sinphi*cosphi

      if (il.eq.0) then
       if (abs(im).eq.0) then
        dth  = (0.d0,0.d0)
        dphi = (0.d0,0.d0)
       else
        stop 'Bug ylm'
       endif
      elseif (il.eq.1) then
       if (abs(im).eq.0) then
        dth  = -dsqrt(3.d0/(4.d0*pig))*sinth
        dphi = (0.d0,0.d0)
       elseif (abs(im).eq.1) then
        dth  = -dsqrt(3.d0/(8.d0*pig))*costh
     &                         *cmplx(cosphi,sinphi)
        dphi = -dsqrt(3.d0/(8.d0*pig))*sinth
     &                         *cmplx(-sinphi,cosphi)
       else
        stop 'Bug ylm'
       endif
      elseif (il.eq.2) then
       if (abs(im).eq.0) then
        dth  = -dsqrt(5.d0/(16.d0*pig))*6.d0*costh*sinth
        dphi = (0.d0,0.d0)
       elseif (abs(im).eq.1) then
        dth  = -dsqrt(15.d0/(8.d0*pig))*(costh**2-sinth**2)
     &                                  *cmplx(cosphi,sinphi)
        dphi = -dsqrt(15.d0/(8.d0*pig))*costh*sinth
     &                      *(0.d0,1.d0)*cmplx(cosphi,sinphi)
       elseif (abs(im).eq.2) then
        dth  = dsqrt(15.d0/(32.d0*pig))*2.d0*costh*sinth
     &                            *cmplx(costwophi,sintwophi)
        dphi = dsqrt(15.d0/(32.d0*pig))*sinth**2
     &                *(0.d0,2.d0)*cmplx(costwophi,sintwophi)
       else
        stop 'Bug ylm'
       endif
      endif


!  treats the case im < 0
      if (im.lt.0) then
       dth  = (-1.d0)**(im)*conjg(dth)
       dphi = (-1.d0)**(im)*conjg(dphi)
      endif

      end subroutine


!     ==================================================================
      real function scpdt(u,v,b1,b2,b3)
!     ==================================================================
!     modified on november the 15th 2000 to obtain vectorialised code
!     ------------------------------------------------------------------
      real u(3), v(3), b1(3), b2(3), b3(3)
!      scpdt = 0.0
!      do i = 1, 3
!        scpdt = scpdt + (u(1)*b1(i) + u(2)*b2(i) + u(3)*b3(i)) *
!     &                  (v(1)*b1(i) + v(2)*b2(i) + v(3)*b3(i)) 
!      enddo
      scpdt = (u(1)*b1(1) + u(2)*b2(1) + u(3)*b3(1)) *
     &        (v(1)*b1(1) + v(2)*b2(1) + v(3)*b3(1)) +
     &        (u(1)*b1(2) + u(2)*b2(2) + u(3)*b3(2)) *
     &        (v(1)*b1(2) + v(2)*b2(2) + v(3)*b3(2)) +
     &        (u(1)*b1(3) + u(2)*b2(3) + u(3)*b3(3)) *
     &        (v(1)*b1(3) + v(2)*b2(3) + v(3)*b3(3)) 
      return
      end
