! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

c Bits 'n' bobs by JPAC 30/3/1989
c -------------------------------
c Subroutines:
c   CLVLDA  - This calculate the density rho and and the exchange-correlation
c             potential
c   VXCCA   - Calculates Ceperly-Alder Vxc
c   VXJAS   - This real function calculates Vx(Rho).This includes the
c             relativistic correction term.
c   VCJAS   - This real function calculates Vc(Rho).
c   KXC     - This calculates the value of Kxc on the real space
c             fourier grid.
c   CLKXC   - This is a routine called by KXC to sort out work space.
c   DIFVXC  - This real fuction calculates Kxc at r=r' for a given charge
c             density.[Kxc(r,r')=dVxc/dn(r) * kronecker(r,r')]
c   DIFFVC  - This real function calculates dVc/dRho for a given charge
c             density.Rs is calculated from the density.
c   DIFFVX  - This real function calculates dVx/dRho for a given charge
c             density.Rs is gained from th e density.
c   VXNR    - This real function calculates Vx(Rho), ignoring relativity.
c   REL     - This calculates the relativistic correction.
c   DIFREL  - This calculates the differential of the relativistic
c             correction
c
c NOTE : All this is for an unpolarised system.For a polarised system
c        you need to change the parameters in VCJAS and DIFFVC (see
c        the references given in the subrutines).


!=======================================================================
      subroutine cmevxclda(nbcmax,
     &  nbnds, NKIBZM, nkibz, nkbz, nop, ninv, nr, nrb,
     &  ucvol, irottb, wtk, ktab,
     &  nkcalc, kcalc, minbnd, maxbnd,
     &  occ, wfr, mevxclda)
!=======================================================================
c     calculate <VxcLDA> for this band (only on first trip)
c     [actually do it for the kIBZ corresponding to this kBZ, but this
c     should be the same since Vxc(r) has the full symmetry]
!-----------------------------------------------------------------------
      implicit none
      integer NBCMAX, NKIBZM, nkibz, nkbz, nop, ninv, nr, nrb, nbnds
      integer irottb(nr,nop)
      real wtk(nkibz)
      integer ktab(nkbz)
      real ucvol
      integer nkcalc, kcalc(nkcalc), minbnd(nkcalc), maxbnd(nkcalc)
      real occ(NKIBZM,*)
      complex wfr(nr,nbnds,NKIBZM)

      complex mevxclda(NBCMAX,nkcalc)

      complex ctemp
      real, allocatable :: vxclda(:)

      integer ik, ib, ibr, ikibz, ir
      integer istat
      logical, parameter :: VERBOSE = .true.


      allocate(vxclda(nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      call cvxclda(
     &  nbnds,NKIBZM,nkibz,nkbz,nop,ninv,nr,nrb,
     &  ucvol,irottb,wtk,
     &  occ,wfr,
     &  vxclda)


      do ik = 1, nkcalc
        ikibz = ktab(kcalc(ik))
        do ib = minbnd(ik), maxbnd(ik)
          ibr = ib - minbnd(ik) + 1
          ctemp = 0.0
          do ir = 1, nr
            ctemp = ctemp + conjg(wfr(ir,ib,ikibz)) *
     &        vxclda(ir) * wfr(ir,ib,ikibz)
          enddo
          mevxclda(ibr,ik) = ctemp / nrb
        enddo
      enddo

      if (VERBOSE) then
        do ik = 1, nkcalc
          write(6,5) ik
5         format(' LDA exchange energies for k-point ', i3/
     &           '  n          <VxcLDA>')
          do ib = minbnd(ik), maxbnd(ik)
            ibr = ib - minbnd(ik) + 1
            write(6,10) ib, mevxclda(ibr,ik) * 27.2116
10          format(i3,2e14.6)
          enddo
        enddo
      endif

      write(*,*)

      deallocate(vxclda)
      return
      end


!=======================================================================
      subroutine cvxclda(
     &  nbnds,NKIBZM,nkibz,nkbz,nop,ninv,nr,nrb,
     &  ucvol,irottb,wtk,
     &  occ,wfr,
     &  vxclda)
!=======================================================================
c     Calculate charge density and VxcLDA
c     Input:
c      wfrs
c     Output:
c      VXCLDA: VxcLDA on FFT grid
!-----------------------------------------------------------------------
      implicit none
      integer NKIBZM, nbnds
      integer nkibz, nkbz, nop, ninv, nr, nrb
      real ucvol
      REAL OCC(NKIBZM,*)
      REAL WTK(NKIBZ), VXCLDA(NR)
      INTEGER IROTTB(nr,NOP)

      complex wfr(nr,nbnds,NKIBZM)

      real rho(nr), rho2(nr)
      real enpuc

      integer ik, ir, ib, iop
      real fact
      real, external :: vxcca


c     print info
      write(*,*) 'calculating charge density and VxcLDA'


c     zero charge density
      do ir = 1, nr
        rho(ir) = 0.0
        rho2(ir) = 0.0
      enddo

c     loop over k-points in IBZ
      do ik = 1, NKIBZ
        do ib = 1, nbnds
c         skip the higher bands if occupation is less than 1E-5
          if (occ(ik,ib).gt.1.0e-5) then
            do ir = 1, nr
              rho(ir) = rho(ir) +
     &          occ(ik,ib)*conjg(wfr(ir,ib,ik))*wfr(ir,ib,ik)*wtk(ik)
            enddo
          endif
        enddo
      enddo

c     loop over symmetry operations, symmetrising rho
c     factor of 2 is for inversion
      fact = real(ninv) / (NKBZ*UCVOL)
      do IOP = 1, NOP
        do IR = 1, NR
          rho2(IR) = rho2(IR) + fact * rho(IROTTB(IR,IOP))
        enddo
      enddo

c     calculate total no. of electrons as a check
      enpuc = 0.0
      do ir = 1, nr
        enpuc = enpuc + rho2(ir)
      enddo
      enpuc = enpuc * ucvol / nrb
      write (6,1113) enpuc
1113  format(' total number of electrons per unit cell = ',f7.4)
      
c     form VxcLDA (in Hartrees)
      do IR = 1, NR
        VXCLDA(IR) = VXCCA(rho2(IR))
      enddo

      RETURN
      END


!=======================================================================
      subroutine ckxclda(nfft1,nfft1a,nfft2,nfft3,
     &                   nr,rho2,Kxclda,NPWMAT,gvec)
!=======================================================================
!     calculate exchange-correlation kernel Kxclda in GG' space
!-----------------------------------------------------------------------
      implicit none
      integer nfft1,nfft1a,nfft2,nfft3
      integer nr, NPWMAT
      real rho2(nr)
      complex Kxclda(NPWMAT,NPWMAT)
      integer gvec(3,NPWMAT)
      complex Kxcldagmg(nr)
      integer ig2,ig3,ig4,ig4x,ig4y,ig4z
!     the same, modx and modx2
      integer :: modx2, i, j
      modx2(i,j) = mod( mod(i,j) + ((1-sign(1,i))/2)*j , j)

!     calculate kernel in G-G' form
      call ckxcldag(nfft1,nfft1a,nfft2,nfft3,nr,rho2,Kxcldagmg)
      do ig2 = 1, NPWMAT
        do ig3 = 1, NPWMAT
          ig4x = modx2(gvec(1,ig3)-gvec(1,ig2),nfft1)
          ig4y = modx2(gvec(2,ig3)-gvec(2,ig2),nfft2)
          ig4z = modx2(gvec(3,ig3)-gvec(3,ig2),nfft3)
          ig4 = 1 + ig4x + ig4y*nfft1a + ig4z*nfft1a*nfft2
          Kxclda(ig3,ig2) = Kxcldagmg(ig4)
        enddo
      enddo
      return
      end

!=======================================================================
      subroutine ckxcldag(nfft1,nfft1a,nfft2,nfft3,
     &                    nr,rho2,Kxclda)
!=======================================================================
!     calculate exchange-correlation kernel Kxclda in G-space
!     put it into Kxclda in igfft form
!-----------------------------------------------------------------------

      integer nfft1,nfft1a,nfft2,nfft3
      integer nr
      real rho2(nr)
      complex Kxclda(nr)
      real fftfct

      call ckxcldar(nr,rho2,Kxclda)

      write(*,*) 'fft Kxclda(r) -> Kxclda(G)'
      write(*,*)

      call c3dfft(Kxclda,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)

      fftfct = 1 / real(nfft1*nfft2*nfft3)
!     do ig = 1, NPWVEC
!       kxcg(ig) = Kxclda(igfft(ig,3,3,3))*fftfct
!     enddo

!     renormalization after fft
      Kxclda(:) = Kxclda(:)*fftfct

      return
      end


c=======================================================================
      subroutine ckxcldar(nr,rho2,Kxclda)
c=======================================================================
c     calculate exchange-correlation kernel Kxclda
c-----------------------------------------------------------------------

      integer nr
      real rho2(nr)
      complex Kxclda(nr)

      write(*,*) 'calculating KxcLDA'
      do ir = 1, nr
        Kxclda(ir) = difvxc(rho2(ir))
      enddo
      write(*,*)
      return
      end



!=======================================================================
      FUNCTION VXCCA(RHO)
!=======================================================================
c     this calculates the exchange correlation energy using Ceperley-Alder
c     and including relativistic effects (PRB 26 P.4199, 1982)
c     rho should be in e/au**3; Vxc is returned in Hartrees
c     * TEMP New version of VXCCA does not overflow if RHO=0
!-----------------------------------------------------------------------

      REAL RHO

      IF (RHO.LT.1.0E-10) THEN
        VXCCA = 0.0
      ELSE
        VXCCA=VXJAS(RHO)+VCJAS(RHO)
      ENDIF
      RETURN
      END


!=======================================================================
      FUNCTION VXJAS(RHO)
!=======================================================================
c     calculate the exchange potential WITH relativistic corrections.
!-----------------------------------------------------------------------
      REAL RHO
      VXJAS=-((3*RHO/3.141592654)**0.3333333333)*REL(RHO)
      RETURN
      END


!=======================================================================
      FUNCTION VCJAS(RHO)
!=======================================================================
c     calculate the correlation potential for a homogenous e- gas,
c     with charge density RHO.Uses the PERDEW & ZUNGER calculations
c     ( See DIFFVC ).
!-----------------------------------------------------------------------
      PARAMETER (PI=3.141592654,ATHIRD=0.33333333333333)
      PARAMETER(A=     0.0311)
      PARAMETER(B=    -0.0480)
      PARAMETER(C=     0.0020)
      PARAMETER(D=    -0.0116)
      PARAMETER(GAMMA=-0.1423)
      PARAMETER(BETA1= 1.0529)
      PARAMETER(BETA2= 0.3334)
      REAL RS,RHO,STOR1,STOR2
c     first calculate Rs from RHO
      RS=(3.0/(4.0*PI*RHO))**ATHIRD
c     need to do different calculations depending on Rs<>1
      IF (RS .GT. 1.0) THEN
       STOR1=(1.0+1.16666667*BETA1*SQRT(RS)+1.3333333*BETA2*RS)
       STOR2=GAMMA*(1.0+BETA1*SQRT(RS)+BETA2*RS)**(-2.0)
       VCJAS=STOR1*STOR2
      ELSE
       STOR1=A*LOG(RS)+(B-ATHIRD*A)
       STOR2=0.6666667*C*RS*LOG(RS)+0.33333333*(2*D-C)*RS
       VCJAS=STOR1+STOR2
      ENDIF
      RETURN
      END



!=======================================================================
      FUNCTION DIFVXC(RHO)
!=======================================================================
c     ****** TEMP Include copy of DIFVXC to print out rho for testing
c     Given the charge density of a homogenous e- gas, this calculates
c     the differential of the Ceperley Alder exchange correlation potential
c     wrt rho (includes relativistic effects see VXCCA).
!-----------------------------------------------------------------------
      SAVE IPRINT
      REAL RHO
      DATA IPRINT/0/
      IF (RHO.LT.1.0E-10) THEN
        IF (IPRINT.NE.1) THEN
          WRITE (6,1030) RHO
1030      FORMAT (//' *** Warning: RHO very small in DIFVXC:',E14.6/
     &              '  This message will not be printed again')
          IPRINT = 1
        ENDIF
        DIFVXC = 0.0
      ELSE
        DIFVXC=DIFFVC(RHO)+DIFFVX(RHO)
      ENDIF
      RETURN
      END


!=======================================================================
      FUNCTION DIFFVC(RHO)
!=======================================================================
c     Given the charge density of a homogenous e- gas, this calculates
c     the differential of the correlation potential wrt rho.
c     It does this by evaluating the differential of the Ceperley-Alder
c     expression given by PERDEW & ZUNGER (PRB,Vol 23,No 10,P.5048,15 May '81)
c     Gamma,Beta1,Beta2,A,B,C,D are parameters for the fit, in atomic
c     units.
!-----------------------------------------------------------------------
      PARAMETER (PI=3.141592654,ATHIRD=0.33333333333333)
      PARAMETER(A=     0.0311)
      PARAMETER(B=    -0.0480)
      PARAMETER(C=     0.0020)
      PARAMETER(D=    -0.0116)
      PARAMETER(GAMMA=-0.1423)
      PARAMETER(BETA1= 1.0529)
      PARAMETER(BETA2= 0.3334)
      REAL RHO,STOR1,STOR2,STOR3,RS
c     calculate Rs from the density
      RS=(3.0/(4.0*PI*RHO))**ATHIRD
c     two different calculations depending if Rs <> 1
      IF (RS .GT. 1.0) THEN
       STOR1=(1.0+BETA1*SQRT(RS)+BETA2*RS)**(-3.0)
       STOR2=-0.41666667*BETA1*(RS**(-0.5))-0.5833333*
     &        (BETA1**2)-0.66666667*BETA2
       STOR3=-1.75*BETA1*BETA2*SQRT(RS)-1.3333333*RS*
     &        (BETA2**2)
       DIFFVC=GAMMA*STOR1*(STOR2+STOR3)
      ELSE
       DIFFVC=A/RS+0.66666667*(C*LOG(RS)+D)+0.33333333*C
      ENDIF
      DIFFVC=DIFFVC*(-4.0*PI/9.0)*(RS**4)
      RETURN
      END


!=======================================================================
      FUNCTION DIFFVX(RHO)
!=======================================================================
c     calculate the differential of the exchange potential wrt Rho for
c     a homogenous e- gas of density RHO.Does this by differenting the
c     relativistic term (see VXCCA).
!-----------------------------------------------------------------------
      PARAMETER (ATHIRD=0.3333333333,PI=3.141592654)
      REAL RHO,RS
      RS=(3.0/(4.0*PI*RHO))**ATHIRD
      DIFFVX=(0.610887057/(RS*RS))*REL(RHO)+
     &       VXNR(RHO)*DIFREL(RHO)
      DIFFVX=DIFFVX*(-4.0*PI/9.0)*(RS**4)
      RETURN
      END


!=======================================================================
      FUNCTION VXNR(RHO)
!=======================================================================
c     calculate the exchange potential WITHOUT any rel corrections.
!-----------------------------------------------------------------------
      REAL RHO
      VXNR=-(3*RHO/3.141592654)**0.3333333333
      RETURN
      END


!=======================================================================
      FUNCTION REL(RHO)
!=======================================================================
c     REL(RHO) calculates the relativistic to Vx.
!-----------------------------------------------------------------------
      PARAMETER (ATHIRD=0.3333333333,PI=3.141592654)
      REAL RHO,RS,B
      RS=(3.0/(4.0*PI*RHO))**ATHIRD
      B=0.0140/RS
      REL=-0.5+1.5*LOG(B+SQRT(1.0+B*B))/(B*SQRT(1.0+B*B))
      RETURN
      END


!=======================================================================
      FUNCTION DIFREL(RHO)
!=======================================================================
c     DIFREL(RHO) calculates dREL/dRs.
!-----------------------------------------------------------------------
      PARAMETER (ATHIRD=0.3333333333,PI=3.141592654)
      REAL RHO,RS,B,BB,BB1
      RS=(3.0/(4.0*PI*RHO))**ATHIRD
      B=0.0140/RS
      BB=B*B
      BB1=1.0+BB
      DIFREL=(1.5/(B*BB1))-1.5*LOG(B+SQRT(BB1))*(1.0+2.0*BB)*
     &       (BB1**(-1.5))/BB
      DIFREL=DIFREL*(-0.0140)/(RS*RS)
      RETURN
      END
